#ifndef OATH_BAG_H

#define OATH_BAG_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  bag (bagA, bagG)
//
//  History:
//    07/91  Brian M Kennedy  utility functions: deepCopy, flatten
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "obj.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class bagA

CLASS_A(bag, obj)

            inline int
        isEmpty () const;

            inline int
        count () const;

            inline int
        contains (const objA& O) const;

	    inline int
        containsEqual (const objA& O) const;

	    inline int
	canContain (const objA& O) const;

	    inline void
	apply (void (*F)(objA)) const;

	    inline const bagA&
	applyX (objA (*F)(objA), const bagA& B) const;

	    inline const bagA&
	applyX (bagA (*F)(objA), const bagA& B) const;


#define bag_RETURNING_DEFINITIONS(type)					    \
	obj_RETURNING_DEFINITIONS(type)					    \
									    \
	    inline const CAT(type,A)&					    \
	insert (const objA& O) const;					    \
									    \
	    inline const CAT(type,A)&					    \
	operator << (const objA& O) const;				    \
									    \
	    inline const CAT(type,A)&					    \
	append (const bagA& B) const;					    \
									    \
	    inline const CAT(type,A)&					    \
	operator << (const bagA& B) const;				    \
									    \
	    inline const CAT(type,A)&					    \
	operator |= (const bagA& B) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator | (const bagA& B) const;				    \
									    \
            inline CAT(type,A)						    \
        makeEmpty () const;						    \
									    \
            inline CAT(type,A)						    \
        make (objA (*F)(objA)) const;					    \
									    \
            inline CAT(type,A)						    \
        make (bagA (*F)(objA)) const;					    \


	bag_RETURNING_DEFINITIONS(bag)

   };


/////////////////////////////////////////////////////////////////////////////
// class bagG

CLASS_G(bag, obj)

	// Constructors //////////

	    inline
        bagG (int IsConst = FALSE);

    public:
        // Bag Operations //////////

	    virtual int
	isEmpty () const 
	    = 0;

	    virtual int
	count () const 
	    = 0;

	    virtual int
	contains (const objG* O) const 
	    = 0;

	    virtual int
	containsEqual (const objG* O) const 
	    = 0;

	    virtual int
	canContain (const objG* O) const 
	    = 0;

	    virtual void
	insert (const objG* O)
	    = 0;

	    virtual void
	append (const bagG* B)
	    = 0;

	    virtual void
	apply (void (*)(objA)) const
	    = 0;

	    virtual bagG*
	applyX (objA (*)(objA), bagG* B) const
	    = 0;

	    virtual bagG*
	applyX (bagA (*)(objA), bagG* B) const
	    = 0;

	    virtual bagA
	makeEmpty () const
	    = 0;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// bagG Inlines

// Constructors //////////

    inline bagG::
bagG (int IsConst) // IsConst = FALSE
   :objG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// bagA Inlines

INLINES(bag, obj)

    inline int bagA::
isEmpty () const
   {return am().guts()->isEmpty();}

    inline int bagA::
count () const
   {return am().guts()->count();}

    inline int bagA::
contains (const objA& O) const
   {return am().guts()->contains(O.guts());}

    inline int bagA::
containsEqual (const objA& O) const
   {return am().guts()->containsEqual(O.guts());}

    inline int bagA::
canContain (const objA& O) const
   {return am().guts()->canContain(O.guts());}

    inline void bagA::
apply (void (*F)(objA)) const
   {am().guts()->apply(F);}

    inline const bagA& bagA::
applyX (objA (*F)(objA), const bagA& B) const
   {am().guts()->applyX(F, B.guts());
    return B;
   }

    inline const bagA& bagA::
applyX (bagA (*F)(objA), const bagA& B) const
   {am().guts()->applyX(F, B.guts());
    return B;
   }


#define bag_RETURNING_INLINES(type)					    \
obj_RETURNING_INLINES(type)						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
insert (const objA& O) const						    \
   {am().guts()->insert(O.guts());					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator << (const objA& O) const					    \
   {return insert(O);}							    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
append (const bagA& B) const						    \
   {am().guts()->append(B.guts());					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator << (const bagA& B) const					    \
   {return append(B);}							    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator |= (const bagA& B) const					    \
   {return append(B);}							    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator | (const bagA& B) const					    \
   {return makeCopy() |= B;}						    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
makeEmpty () const							    \
   {return (CAT(type,A)&)(am().guts()->makeEmpty());}			    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
make (objA (*F)(objA)) const						    \
   {CAT(type,A) Temp = makeEmpty();					    \
    applyX(F, Temp);							    \
    return Temp;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
make (bagA (*F)(objA)) const						    \
   {CAT(type,A) Temp = makeEmpty();					    \
    applyX(F, Temp);							    \
    return Temp;							    \
   }									    \


bag_RETURNING_INLINES(bag)


/////////////////////////////////////////////////////////////////////////////
// Utility functions

    objA
deepCopy (objA O);

    bagA
flatten (objA O);


//***************************************************************************

#endif /* OATH_BAG_H */
