#ifndef OATH_CHARACTER_H

#define OATH_CHARACTER_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  character (characterA, characterG)
//
//  History:
//    06/92  Brian M Kennedy  Made TheCharacters array a non-member (porting)
//    07/91  Brian M Kennedy  import, export, typeRegister
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//			      Removed unnecessary inlines from characterG
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "token.h"

#include <ctype.h>

#define OATH_CHAR_CNT 256  /* CHAR_MAX - CHAR_MIN + 1 */

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class characterA

CLASS_A(character, token)

	// Makes //////////

            inline static characterA
        make (unsigned char);


	// Operations //////////

	    inline unsigned char
	value () const;

	    inline int
	operator <= (const characterA& C) const;

	    inline int
	operator <  (const characterA& C) const;

	    inline int
	operator >  (const characterA& C) const;

	    inline int
	operator >= (const characterA& C) const;

	    inline int
	isSpace () const;

	    inline int
	isLetter () const;

	    inline int
	isDigit () const;

	    inline int
	isHexDigit () const;

	    inline int
	isAlphaNum () const;

	    inline int
	isUppercase () const;

	    inline int
	isLowercase () const;

	    inline int
	isPunctuation () const;

	    inline int
	isPrinting () const;

	    inline int
	isGraphic () const;

	    inline int
	isControl () const;

	    inline const char*
	name () const;


#define character_RETURNING_DEFINITIONS(type)				    \
	token_RETURNING_DEFINITIONS(type)				    \
									    \
	    inline CAT(type,A)						    \
	lowercase () const;						    \
									    \
	    inline CAT(type,A)						    \
	uppercase () const;						    \


	character_RETURNING_DEFINITIONS(character)

   };


// Other Operations //////////

    ostream&
operator << (ostream& F, const characterA& C);

    istream&
operator >> (istream& F, characterA& C);


/////////////////////////////////////////////////////////////////////////////
// class characterG

CLASS_G(character, token)

        // Members //////////

            unsigned char	 	// the C unsigned char value of this
        Value;				// character


	// Constructors //////////

	    virtual
	~characterG ();


    public:
        characterG ();  // Constructs a new characterG 
		        // corresponding to the next char
			// Made public so that it can be called from the
			// static initializer at global scope.  However,
			// it should not be publicly accessible!

        // oathCore Operations //////////

            inline virtual int
        isImplementedAs (void* TypeId) const;

	    virtual void
	export (exportP& X) const;

	    static objA
	import (importP& M);

        // obj Operations //////////

        // character Operations //////////

            inline unsigned char
        value () const;

            inline const char *		// ASCII dependent
        name () const;

	    inline const characterG*
        lowercase () const;

	    inline const characterG*
        uppercase () const;


    private:

        // Static Members //////////

	    static unsigned char	 // contains char value of next
        Count;				 // character to be constructed

	    static int			 // TRUE if 256 characters constructed
        AllConstructed;			

	    static const characterG* 	 // indexed by a C unsigned char value,
	ValueToCharacters[OATH_CHAR_CNT];// this array will yield a  pointer
					 // to the corresponding characterG

            static const char *		 // indexed by a character's value,
        CharacterNames[OATH_CHAR_CNT];	 // this array yields the corresponding
					 // Name (a char*) ASCII dependent!!

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// characterG Inlines

// oathCore Operations //////////

    inline int characterG::      
isImplementedAs (void* TypeID) const
   {return TypeID == Type;}

// character Operations //////////

    inline unsigned char characterG::
value () const
   {return Value;}

    inline const char * characterG::   // ASCII dependent
name () const
   {return CharacterNames[Value];}

    inline const characterG* characterG::
lowercase () const
   {return ValueToCharacters[tolower(Value)];}

    inline const characterG* characterG::
uppercase () const
   {return ValueToCharacters[toupper(Value)];}


/////////////////////////////////////////////////////////////////////////////
// characterA Inlines

INLINES(character, token)

// Makes //////////

    inline characterA characterA::
make (unsigned char C)
   {return characterG::ValueToCharacters[C];}


// Operations //////////

    inline unsigned char characterA::
value () const
   {return am().guts()->value();}

    inline const char* characterA::
name () const
   {return am().guts()->name();}

    inline int characterA::
operator <= (const characterA& C) const
   {return value() <= C.value();}

    inline int characterA::
operator <  (const characterA& C) const
   {return value() < C.value();}

    inline int characterA::
operator >  (const characterA& C) const
   {return value() > C.value();}

    inline int characterA::
operator >= (const characterA& C) const
   {return value() >= C.value();}

    inline int characterA::
isSpace () const
   {return isspace(value());}

    inline int characterA::
isLetter () const
   {return isalpha(value());}

    inline int characterA::
isDigit () const
   {return isdigit(value());}

    inline int characterA::
isHexDigit () const
   {return isxdigit(value());}

    inline int characterA::
isAlphaNum () const
   {return isalnum(value());}

    inline int characterA::
isUppercase () const
   {return isupper(value());}

    inline int characterA::
isLowercase () const
   {return islower(value());}

    inline int characterA::
isPunctuation () const
   {return ispunct(value());}

    inline int characterA::
isPrinting () const
   {return isprint(value());}

    inline int characterA::
isGraphic () const
   {return isgraph(value());}

    inline int characterA::
isControl () const
   {return iscntrl(value());}


#define character_RETURNING_INLINES(type)				    \
token_RETURNING_INLINES(type)						    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
lowercase () const							    \
   {return am().guts()->lowercase();}					    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
uppercase () const							    \
   {return am().guts()->uppercase();}					    \


character_RETURNING_INLINES(character)


//***************************************************************************

#endif /* OATH_CHARACTER_H */
