#ifndef OATH_DEQ_H

#define OATH_DEQ_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  deq (deqA, deqG)
//
//  History:
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "fifoQueue.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class deqA

CLASS_A(deq, fifoQueue)

            inline objA
        end () const;


#define deq_RETURNING_DEFINITIONS(type)			    	    	    \
	fifoQueue_RETURNING_DEFINITIONS(type)				    \
									    \
            inline const CAT(type,A)&					    \
        pushFront (const objA& O) const;				    \
									    \
            inline const CAT(type,A)&					    \
        pushEnd (const objA& O) const;					    \
									    \
            inline const CAT(type,A)&					    \
        pullFrontX (objA& O) const;					    \
									    \
            inline const CAT(type,A)&					    \
        pullFront () const;						    \
									    \
            inline const CAT(type,A)&					    \
        pullEndX (objA& O) const;					    \
									    \
            inline const CAT(type,A)&					    \
        pullEnd () const;						    \


	deq_RETURNING_DEFINITIONS(deq)

   };


/////////////////////////////////////////////////////////////////////////////
// class deqG

CLASS_G(deq, fifoQueue)

	// Constructors //////////

	    inline
        deqG (int IsConst = FALSE);

    public:
        // Deq Operations //////////

	    virtual const objG*
	end () const
	    = 0;

	    virtual void
	pushFront (const objG* O)
	    = 0;

	    virtual const objG*
	pullEnd ()
	    = 0;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// deqG Inlines

// Constructors //////////

    inline deqG::
deqG (int IsConst) // IsConst = FALSE)
   :fifoQueueG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// deqA Inlines

INLINES(deq, fifoQueue)

#define deq_RETURNING_INLINES(type)				    	    \
fifoQueue_RETURNING_INLINES(type)					    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
pushFront (const objA& O) const						    \
   {am().guts()->pushFront(O.guts());					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
pushEnd (const objA& O) const						    \
   {return insert(O);}							    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
pullFrontX (objA& O) const						    \
   {return extractX(O);}						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
pullFront () const							    \
   {return extract();}							    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
pullEndX (objA& O) const						    \
   {O = am().guts()->pullEnd();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
pullEnd () const							    \
   {am().guts()->pullEnd();						    \
    return *this;							    \
   }									    \


deq_RETURNING_INLINES(deq)


    inline objA deqA::
end () const
   {return am().guts()->end();}


//***************************************************************************

#endif /* OATH_DEQ_H */
