#ifndef OATH_DLLIST_H

#define OATH_DLLIST_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  dlList (dlListA, dlListG)
//  dlPos (dlPosA, dlPosG)
//
//  History:
//    07/91  Brian M Kennedy  import, export, typeRegister
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "list.h"

#include "dlNodeP.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

class dlPosA;
class dlPosG;

/////////////////////////////////////////////////////////////////////////////
// class dlListA

CLASS_A(dlList, list)

	// Makes //////////

            static inline dlListA
        make (int MakeConst = FALSE);

            static inline dlListA
        make (const seqA& S, int MakeConst = FALSE);

            static inline dlListA
        make (const posA& S, const posA& B, int MakeConst = FALSE);


	// Operations //////////

	    inline dlPosA
	makePos (int I = 0, int MakeConst = FALSE) const;

	list_RETURNING_DEFINITIONS(dlList)

   };


/////////////////////////////////////////////////////////////////////////////
// class dlListG

CLASS_G(dlList, list)

    friend class dlPosG;

	// Members //////////

    	dlNodeP * Header;	// should be dlNodeAT const *
    	dlNodeP * End;
	dlPosG *  PosList;


	// Constructors //////////

        dlListG (int IsConst = FALSE);

	    inline
        dlListG (dlNodeP* iHeader, dlNodeP* iEnd, int IsConst = FALSE);

        dlListG (const seqG* S, int IsConst = FALSE);

        dlListG (const posG* Start, const posG* Beyond, int IsConst = FALSE);

        ~dlListG ();

    public:
        // oathCore Operations //////////

            inline virtual int
        isImplementedAs (void* TypeId) const;

	    virtual void
	export (exportP& X) const;

	    static objA
	import (importP& M);

            virtual void
	clearReferences ();

            virtual void
	setReferences ();


        // obj Operations //////////

            virtual int
        isEqual (const objG* O) const;

            virtual objA
        makeCopy (int MakeConst) const;


        // bag Operations //////////

            inline virtual int
        isEmpty () const;

            virtual int
        count () const;

            virtual int
        contains (const objG* O) const;

            virtual int
        containsEqual (const objG* O) const;

            virtual int
        canContain (const objG* O) const;

            virtual void
        insert (const objG* O);

            virtual void
        append (const bagG* B);

            virtual void
        apply (void (*F)(objA)) const;

            virtual bagG*
        applyX (objA (*F)(objA), bagG* B) const;

            virtual bagG*
        applyX (bagA (*F)(objA), bagG* B) const;

            virtual bagA
        makeEmpty () const;


        // queue Operations //////////

	    inline virtual const objG*
	front () const;

	    virtual const objG*
	remove ();


        // seq Operations //////////

            virtual const objG*
        subscript (int I) const;

            virtual const objG*
        subscript (const posG* P) const;

            inline virtual posA
        makePos (int I, int MakeConst) const;

	    virtual seqA
        makeSeq (const posG* Start, const posG* Beyond, int MakeConst) const;

	    virtual seqA
        makeSeq (int Start, int Beyond, int MakeConst) const;


        // fifoQueue Operations //////////


        // deq Operations //////////

            inline virtual const objG*
        end () const;

            virtual void
        pushFront (const objG* O);

            virtual const objG*
        pullEnd ();


        // list Operations //////////

	    virtual void
	insertList (const listPosG*, const objG*);

	    virtual const objG*
	removeList (const listPosG*);


        // dlList Operations //////////

            dlNodeP *
        internalPosition (int I) const;

	    inline dlPosG*&
	posList () const;	// not really const!!!

	    void
	adjustPosList (dlNodeP * CurrentPrev, dlNodeP * NewPrev);

	    inline void
	adjustEnd (dlNodeP * OldEnd, dlNodeP * NewEnd) const; // Not really!!

   };


/////////////////////////////////////////////////////////////////////////////
// class dlPosA

CLASS_A(dlPos, listPos)

	listPos_RETURNING_DEFINITIONS(dlPos)

   };


/////////////////////////////////////////////////////////////////////////////
// class dlPosG

CLASS_G(dlPos, listPos)

    friend class dlListG;

	// Members //////////

        const dlListA List;
        dlNodeP *     Prev;
	dlPosG *      PrevPos;
	dlPosG *      NextPos;

	// Constructors //////////

	    inline
        dlPosG (int IsConst = FALSE);

	dlPosG (const dlListG* iList, dlNodeP* iPrev, int IsConst = FALSE);

	~dlPosG ();

    public:
        // oathCore Operations //////////

            inline virtual int
        isImplementedAs (void* TypeId) const;

	    virtual void
	export (exportP& X) const;

	    static objA
	import (importP& M);

            virtual void
	clearReferences ();

            virtual void
	setReferences ();


        // obj Operations //////////

            virtual int
        isEqual (const objG* O) const;

            inline virtual objA
        makeCopy (int MakeConst) const;


        // pos Operations //////////

            inline virtual int
        isPastEnd () const;

            virtual const objG*
        indirection () const;

            virtual void
        increment ();

            virtual void
        increment (int I);

            virtual const objG*
        find (const objG* O);

            virtual const objG*
        findEqual (const objG* O);

            virtual void
        reset (const posG* P);

            virtual void
        reset (int I);

            inline virtual const seqG*
        parent () const;


        // listPos Operations //////////

            virtual void
        decrement ();

            virtual void
        decrement (int I);

        // dlPos Operations //////////

	    inline dlNodeP*&
	prev () const; // Not really const!!

	    inline virtual void
	adjustEnd (dlNodeP * NewEnd) const; // Not really const!!

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// dlListG Inlines

// Constructors //////////

    inline dlListG::
dlListG (dlNodeP* iHeader, dlNodeP* iEnd, int IsConst)
   :listG(IsConst), Header(iHeader), End(iEnd), PosList(0)
   {}


// oathCore Operations //////////

    inline int dlListG::
isImplementedAs (void* TypeId) const
   {return TypeId == Type;}


// bag Operations //////////

    inline int dlListG::
isEmpty () const
   {return !Header->next();}

// queue Operations //////////

    inline const objG* dlListG::
front () const
   {assumed(!isEmpty(), "front() attempted on an empty deq!");
    return Header->nextObj().guts();
   }


// seq Operations //////////

    inline posA dlListG::
makePos (int I, int MakeConst) const
   {return new dlPosG (this, internalPosition(I), MakeConst);}


// fifoQueue Operations //////////


// deq Operations //////////

    inline const objG* dlListG::
end () const
   {assumed(!isEmpty(), "end() attempted on an empty deq!");
    return End->thisObj().guts();
   }


// list Operations //////////

// dlList Operations //////////

    inline dlPosG*& dlListG::
posList () const	// not really const!!!
   {return ((dlListG*)this)->PosList;}

    inline void dlListG::
adjustEnd (dlNodeP * OldEnd, dlNodeP * NewEnd) const // Not really!!
   {if(End == OldEnd)
	((dlListG*)this)->End = NewEnd;
   }


/////////////////////////////////////////////////////////////////////////////
// dlListA Inlines

INLINES(dlList, list)

    inline dlListA dlListA::
make (int MakeConst)
   {return new dlListG(MakeConst);}

    inline dlListA dlListA::
make (const seqA& S, int MakeConst)
   {return new dlListG(S.guts(), MakeConst);}

    inline dlListA dlListA::
make (const posA& S, const posA& B, int MakeConst)
   {return new dlListG(S.guts(), B.guts(), MakeConst);}


    inline dlPosA dlListA::
makePos (int I, int MakeConst) const
   {return (dlPosA&)(am().guts()->makePos(I, MakeConst));}


list_RETURNING_INLINES(dlList)


/////////////////////////////////////////////////////////////////////////////
// dlPosG Inlines

// Constructors //////////

    inline dlPosG::
dlPosG (int IsConst)
   :listPosG(IsConst)
   {}


// oathCore Operations //////////

    inline int dlPosG::
isImplementedAs (void* TypeId) const
   {return TypeId == Type;}

// obj Operations //////////

    inline objA dlPosG::
makeCopy (int MakeConst) const
   {return new dlPosG(List.guts(), Prev, MakeConst);}


// pos Operations //////////

    inline int dlPosG::
isPastEnd () const
   {return !Prev->next();}

    inline const seqG* dlPosG::
parent () const
   {return List.guts();}


// listPos Operations //////////


// dlPos Operations //////////

    inline dlNodeP*& dlPosG::
prev () const // Not really const!!
   {return ((dlPosG*)this)->Prev;}

    inline void dlPosG::
adjustEnd (dlNodeP * NewEnd) const // Not really const!!
   {List.guts()->adjustEnd(Prev, NewEnd);
    ((dlPosG*)this)->Prev = NewEnd;
   }


/////////////////////////////////////////////////////////////////////////////
// dlPosA Inlines

INLINES(dlPos, listPos)

listPos_RETURNING_INLINES(dlPos)


//***************************************************************************

#endif	/* OATH_DLLIST_H */
