#ifndef OATH_DLCNODEP_H

#define OATH_DLCNODEP_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  dlcNodeP
//
//  History:
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************
//  dlcNodeP (Doubly-Linked Character Node Auxiliary Type)
//
//  dlcNodeP is an auxiliary structure used to implement the dlString type.
//  dlcNodeP is NOT an obj!  dlcNodeP is a class containing two data items:
//  a const_characterD and pointers to the next and previous dlcNodeP's.
//
//  The following operations are defined on dlcNodeP's:
//    prev()		=== Return a pointer to the previous dlcNodeP.
//    thisObj()         === Return a pointer to the characterG in this dlcNodeP.
//    next()		=== Return a pointer to the next dlcNodeP.
//    nextObj()         === Return a pointer to the characterG in the next
//			    dlcNodeP.
//    insertAfter(Ch)   === Make and insert a dlcNodeP pointing to Ch as the
//			    next node.
//    removeNext()	=== Remove and return the next dlcNodeP.
//    deleteNext()	=== Remove and delete the next dlcNodeP.
//
//***************************************************************************

#include "character.h"

/////////////////////////////////////////////////////////////////////////////
// dlcNodeP Class Definition

class dlcNodeP

   {public:

   	dlcNodeP*  Prev;
   	dlcNodeP*  Next;
	characterA Obj;

   	dlcNodeP ()
	   :Prev(0), Next(0), Obj()
	   {}

	dlcNodeP (dlcNodeP* iPrev, dlcNodeP* iNext, const characterG* iObj)
	   :Prev(iPrev), Next(iNext), Obj(iObj)
	   {}

	    dlcNodeP*
	prev ()
	   {return Prev;}

	    dlcNodeP*
	next ()
	   {return Next;}

	    characterA&
	thisObj ()
	   {return Obj;}

	    characterA&
	nextObj ()
	   {assumed(Next, "Attempted dlcNodeP::nextObj() with no Next"); 
	    return Next->Obj;
	   }

	    dlcNodeP*
	insertAfter (const characterG* NewObj)
	   {Next = new dlcNodeP (this, Next, NewObj);
	    if(Next->Next)
		Next->Next->Prev = Next;
	    return Next;
	   }

	    dlcNodeP*
	removeNext ()
	   {assumed(Next, "Attempted dlcNodeP::removeNext() with no Next");
	    dlcNodeP *DLN = Next;
	    Next = Next->Next;
	    if(Next)
		Next->Prev = this;
	    return DLN;
	   }

	    void
	deleteNext ()
	   {assumed(Next, "Attempted dlcNodeP::deleteNext() with no Next");
	    dlcNodeP *DLN = Next;
	    Next = Next->Next;
	    if(Next)
		Next->Prev = this;
	    delete DLN;
	   }

    // Dynamic Allocation

#ifdef OATH_HANDLE_NEW

	static int       AllocateCount;
	static dlcNodeP *FreeList;

	    static void
	allocate ();

	    static void*
        operator new (size_t)
           {if(!FreeList)
	       allocate(); 
	    dlcNodeP *DLN = FreeList;
	    FreeList = FreeList->Next;
            return DLN;
           }

            static void
        operator delete (void *DLN, size_t)
           {((dlcNodeP*)DLN)->Next = FreeList;
            FreeList = (dlcNodeP*)DLN;
           }

#endif

   };

/***************************************************************************/

#endif  /* OATH_DLCNODEP_H */
