#ifndef EXPORTP_H

#define EXPORTP_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  ExportP
//
//  History:
//    07/91  Brian M Kennedy  Original
//
//***************************************************************************
//  ExportP (exportP, importP, objRegisterP, oidP)
//
//  ExportP is an auxiliary structure used to implement the export/import
//  functionality.  ExportP is NOT an obj!  ExportP is a class containing
//
//  The following operations are defined on ExportP's:
//
//***************************************************************************

#include "typeRegisterP.h"

typedef long iosFlags;		// documented as int??

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

typedef unsigned int oidP;

/////////////////////////////////////////////////////////////////////////////
// objRegisterP Class Definition

class objRegisterP
   {friend class exportP;
    friend class importP;

        // Members //////////

	const oathCoreG*    Obj;
	const oidP          OID;
   	const objRegisterP* Next;

        // Constructors //////////

   	objRegisterP ()
	   :Obj(0), OID(0), Next(0)
	   {ensure(0, "This shouldn't be called!");}

	objRegisterP (const objRegisterP&)
	   :Obj(0), OID(0), Next(0)
	   {ensure(0, "This shouldn't be called!");}

	objRegisterP (const oathCoreG* O, oidP I, objRegisterP* N)
	   :Obj(O), OID(I), Next(N)
	   {}

	// Operations //////////

	    oidP
        oid (const oathCoreG* O) const
	   {return O == Obj ? OID : (Next ? Next->oid(O) : 0);}

	    const oathCoreG*
        obj (oidP I) const
	   {return I == OID ? Obj : (Next ? Next->obj(I) : 0);}

   };


/////////////////////////////////////////////////////////////////////////////
// exportP Class Definition

class exportP
   {friend class oathCoreA;
    friend class oathCoreG;

        // Members //////////

	oidP		LastOID;
	unsigned long   HashMask;
   	objRegisterP ** Table;
	ostream&   	Stream;
        iosFlags        StreamFlags;

        // Constructors //////////

   	exportP ();

	exportP (const exportP&);

	exportP (ostream& S, unsigned int HashKey = 6);

	~exportP ();

    public:
	// Operations //////////

	    ostream&
	stream () const
	   {return Stream;}

	    void
	writeType (const char* TypeName)
	   {typeRegisterP::writeType(Stream, TypeName);}

    private:
	    int
        registerObj (const oathCoreG* O, oidP& I);

   };


/////////////////////////////////////////////////////////////////////////////
// importP Class Definition

class importP
   {friend class oathCoreA;
    friend class oathCoreG;

        // Members //////////

	unsigned long   HashMask;
   	objRegisterP ** Table;
	istream&   	Stream;
        iosFlags	StreamFlags;

        // Constructors //////////

   	importP ();

	importP (const importP&);

	importP (istream& S, unsigned int HashKey = 6);

   	~importP ();

    public:
	// Operations //////////

	    istream&
	stream () const
	   {return Stream;}

	    const char*
	readType ()
	   {return typeRegisterP::readType(Stream);}

    private:
	    const oathCoreG*
	obj (oidP I)
	   {unsigned long Hash = HashMask & I;
	    return (Table[Hash] ? Table[Hash]->obj(I) : 0);
	   }

	    void
        registerObj (const oathCoreG* O, oidP I)
	   {unsigned long Hash = HashMask & I;
	    Table[Hash] = new objRegisterP (O, I, Table[Hash]);
	   }

   };


/***************************************************************************/

#endif  /* EXPORTP_H */
