#ifndef HASHNODEP_H

#define HASHNODEP_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  hashNodeP
//
//  History:
//    04/91  Brian M Kennedy  Original
//
//***************************************************************************
//  hashNodeP (String Table Node)
//
//  hashNodeP is an auxiliary structure used to implement the hashSet
//  type.  hashNodeP is NOT an obj!  hashNodeP is a class containing
//  three data items:  a const_minStringA, a const_objA, and a pointer to
//  the next hashNodeP.
//
//  The following operations are defined on hashNodeP's:
//    contains(S)	--- Return TRUE if this const_stringTokenA is equal to
//			    the stringToken S or if Next->contains(S).
//    assoc(S)		--- Return this const_objA if this const_stringTokenA is
//			    equal to the stringToken S or return Next->assoc(S).
//    reassoc(S, O)	--- If this const_stringTokenA is equal to S, then 
//			    change this const_objA to O and return TRUE.
//			    Otherwise, if Next, return Next->reassoc(S, O).
//    lookup(Str)	--- If the string Str is equal to this stringToken's
//			    string, then return this stringToken.  Otherwise
//			    return Next->lookup(Str).
//    lookup(Chars)     --- Same as above for char* Chars.
//    lookup(Start, Bey)--- Same as above for stringPos's Start and Bey.
//
//
//***************************************************************************

class hashSetG;

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// hashNodeP Class Definition

class hashNodeP
   {friend class hashSetG;

        // Members //////////

	objA        Obj;
   	hashNodeP * Next;

        // Constructors //////////

   	hashNodeP ()
	   :Next(0), Obj()
	   {ensure(FALSE, "This shouldn't be called!");}

	hashNodeP (const hashNodeP&)
	   :Next(0), Obj()
	   {ensure(FALSE, "This shouldn't be called!");}

	hashNodeP (const objG* O, hashNodeP* N)
	   :Next(N), Obj(O)
	   {}

	// Operations //////////

	    int
        contains (const objG* O) const
	   {return O->is(Obj.guts()) 
		   ? TRUE
		   : (Next ? Next->contains(O) : FALSE);
	   }

	    void
	remove (const objG* O, hashNodeP*& P)
	   {if(O->is(Obj.guts()))
	       {P = Next;
		delete this;
	       }
	    else if(Next)
	        Next->remove(O, Next);
	   }

   };

/***************************************************************************/

#endif  /* HASHNODEP_H */
