#ifndef OATH_HASHSET_H

#define OATH_HASHSET_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  hashSet (hashSetA, hashSetG)
//
//  History:
//    07/91  Brian M Kennedy  import, export, typeRegister
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    04/91  Brian M Kennedy  Original
//
//***************************************************************************

#include "set.h"

#include "hashNodeP.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class hashSetA

CLASS_A(hashSet, set)

	    static inline hashSetA
	make (unsigned int Buckets = 255, int MakeConst = FALSE);

	set_RETURNING_DEFINITIONS(hashSet)

   };


/////////////////////////////////////////////////////////////////////////////
// class hashSetG

CLASS_G(hashSet, set)

    friend class stringTokenG;

	// Members //////////

        hashNodeP**  Set;
	unsigned int Count;
	unsigned int Buckets;

	// Constructors //////////

        hashSetG (unsigned int IBuckets = 1, int IsConst = FALSE);

    public:
        // oathCore Operations //////////

            inline virtual int
        isImplementedAs (void* TypeId) const;

	    virtual void
	export (exportP& X) const;

	    static objA
	import (importP& M);

            virtual void
	clearReferences ();

            virtual void
	setReferences ();


        // obj Operations //////////

            virtual
        isEqual (const objG* O) const;

            virtual objA
        makeCopy (int MakeConst) const;


        // bag Operations //////////

            inline virtual int
        isEmpty () const;

            inline virtual int
        count () const;

            virtual int
        contains (const objG* O) const;

            virtual int
        containsEqual (const objG* O) const;

            virtual int
        canContain (const objG* O) const;

            virtual void
        insert (const objG* O);

            virtual void
        append (const bagG* B);

	    virtual void
	apply (void (*)(objA)) const;

	    virtual bagG*
	applyX (objA (*)(objA), bagG* B) const;

	    virtual bagG*
	applyX (bagA (*)(objA), bagG* B) const;

	    virtual bagA
	makeEmpty () const;

        // Set Operations //////////

	    virtual void
	remove (const objG* O);

	    virtual void
	subtract (const setG* S);

	    virtual void
	intersect (const setG* S);

        // hashSet Operations //////////

	    inline unsigned int
	hash (const objG* O) const;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// hashSetG Inlines

// oathCore Operations //////////

    inline int hashSetG::
isImplementedAs (void* TypeId) const
   {return TypeId == Type;}


// bag Operations //////////

    inline int hashSetG::
isEmpty () const
   {return !Count;}

    inline int hashSetG::
count () const
   {return Count;}


// hashSet Operations //////////

    inline unsigned int hashSetG::
hash (const objG* O) const
   {unsigned long HV = (unsigned long)(void*)O;
    return (HV >> 3) % Buckets;
   }


/////////////////////////////////////////////////////////////////////////////
// hashSetA Inlines

INLINES(hashSet, set)

    inline hashSetA hashSetA::
make (unsigned int Buckets, int MakeConst)
   {return new hashSetG (Buckets, MakeConst);}


set_RETURNING_INLINES(hashSet)


//***************************************************************************

#endif /* OATH_HASHSET_H */
