#ifndef OATH_LIST_H

#define OATH_LIST_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  list (listA, listG)
//  listPos (listPosA, listPosG)
//
//  History:
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "deq.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

class listPosA;
class listPosG;

/////////////////////////////////////////////////////////////////////////////
// class listA

CLASS_A(list, deq)

	    inline objA
        remove (const listPosA& P) const;

	    inline listPosA
	makePos (int I = 0, int MakeConst = FALSE) const;


#define list_RETURNING_DEFINITIONS(type)		    	    	    \
	deq_RETURNING_DEFINITIONS(type)					    \
									    \
            inline const CAT(type,A)&					    \
        insert (const listPosA& P, const objA& O) const;		    \
									    \
	    inline const CAT(type,A)&					    \
        extractX (const listPosA& P, objA& O) const;			    \
									    \
	    inline const CAT(type,A)&					    \
        extract (const listPosA& P) const;				    \


	list_RETURNING_DEFINITIONS(list)

   };


/////////////////////////////////////////////////////////////////////////////
// class listG

CLASS_G(list, deq)

    friend listPosG;

	// Constructors //////////

	    inline
        listG (int IsConst = FALSE);

    public:
        // List Operations //////////

	    virtual void
	insertList (const listPosG*, const objG*)
	    = 0;

	    virtual const objG*
	removeList (const listPosG*)
	    = 0;

   };


/////////////////////////////////////////////////////////////////////////////
// class listPosA

CLASS_A(listPos, pos)


#define listPos_RETURNING_DEFINITIONS(type)		    	    	    \
	pos_RETURNING_DEFINITIONS(type)					    \
									    \
            inline const CAT(type,A)&					    \
	operator -- () const;						    \
									    \
	    inline const CAT(type,A)&					    \
	operator -= (int I) const;					    \


	listPos_RETURNING_DEFINITIONS(listPos)

   };


/////////////////////////////////////////////////////////////////////////////
// class listPosG

CLASS_G(listPos, pos)

    friend listG;

	// Constructors //////////

	    inline
        listPosG (int IsConst = FALSE);

    public:
        // listPos Operations //////////

	    virtual void
	decrement ()
	    = 0;
	    
	    virtual void
	decrement (int I)
	    = 0;
	    
   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// listG Inlines

// Constructors //////////

    inline listG::
listG (int IsConst) // IsConst = FALSE)
   :deqG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// listA Inlines

INLINES(list, deq)

    inline objA listA::
remove (const listPosA& P) const
   {return am().guts()->removeList(P.guts());}

    inline listPosA listA::
makePos (int I, int MakeConst) const
   {return (listPosA&)(am().guts()->makePos(I, MakeConst));}


#define list_RETURNING_INLINES(type)				    	    \
deq_RETURNING_INLINES(type)						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
insert (const listPosA& P, const objA& O) const				    \
   {am().guts()->insertList(P.guts(), O.guts());			    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
extractX (const listPosA& P, objA& O) const				    \
   {O = am().guts()->removeList(P.guts());				    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
extract (const listPosA& P) const					    \
   {am().guts()->removeList(P.guts());					    \
    return *this;							    \
   }									    \


list_RETURNING_INLINES(list)


/////////////////////////////////////////////////////////////////////////////
// listPosG Inlines

// Constructors //////////

    inline listPosG::
listPosG (int IsConst) // IsConst = FALSE)
   :posG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// listPosA Inlines

INLINES(listPos, pos)


#define listPos_RETURNING_INLINES(type)				    	    \
pos_RETURNING_INLINES(type)						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator -- () const							    \
   {am().guts()->decrement();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator -= (int I) const						    \
   {am().guts()->decrement(I);						    \
    return *this;							    \
   }									    \


listPos_RETURNING_INLINES(listPos)


//***************************************************************************

#endif	/* OATH_LIST_H */
