#ifndef OATH_MACROS_H

#define OATH_MACROS_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  MACRO Definitions for OATH
//
//  History:
//    08/91  Brian M Kennedy  removed G copy constructor/assign bodies
//    07/91  Brian M Kennedy  import, export, typeRegister
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

/////////////////////////////////////////////////////////////////////////////
// be() and am()

#define be()  ensureSelf(__FILE__, __LINE__)


#ifdef NDEBUG

#define am()  self()

#else

#define am()  ensureSelf(__FILE__, __LINE__)

#endif // NDEBUG


/////////////////////////////////////////////////////////////////////////////
// Concatenation and String Operations (cpp dependent)

#ifdef __STDC__
#define CAT(type,ext) type##ext
#define STRING(str)  #str
#else
#define CAT(type,ext) type/**/ext
#define STRING(str)  "str"
#endif


/////////////////////////////////////////////////////////////////////////////
// class xxxA header

#define CLASS_AAG(typeA, ParentA, typeG)       	    		    	    \
class typeG;								    \
									    \
class typeA								    \
   :public ParentA							    \
   {public:                                                                 \
        /* Static Members */						    \
	static const typeA Nil;						    \
                                                                            \
	/* Static Operations */						    \
	    static inline const typeA&					    \
	isa (const oathCoreA& O);					    \
                                                                            \
            static inline const typeA&					    \
	isa (const typeA& O);						    \
									    \
	/* Constructors */						    \
        inline typeA ();						    \
                                                                            \
        inline typeA (const typeG* iGuts);				    \
                                                                            \
        inline typeA (const typeA& O);					    \
                                                                            \
        /* Operations */						    \
        inline typeA& operator = (const typeA& O);			    \
                                                                            \
        inline typeG * guts () const;					    \
                                                                            \
        /* Makes and Operations */					    \


#define CLASS_A(type, parent)						    \
CLASS_AAG(CAT(type,A), CAT(parent,A), CAT(type,G))			    \


/////////////////////////////////////////////////////////////////////////////
// class xxxG header

#define CLASS_GGA(typeG, parentG, typeA)                                    \
class typeG								    \
   :public parentG							    \
   {friend class typeA;							    \
									    \
    /* G classes should be nested classes of their corresponding A's */	    \
    /* but that is not currently possible due to compiler weakness.  */	    \
    /* Until then all G members appear readily accessible.           */	    \
    /* Please do not access a G member from outside another G member */	    \
    /* or an A member.                                               */	    \
									    \
    private:								    \
        /* Bogus outline function to help some compilers properly */        \
	/* position a single virtual function table               */        \
            virtual void						    \
        vtablePosition () const;					    \
									    \
        /* Hide default constructor and assignment operator */		    \
        typeG (const typeG&);						    \
									    \
	    typeG&							    \
	operator= (const typeG&);					    \
									    \
    public:								    \
        /* Static Members */						    \
									    \
	static void*       Type;  /* The Type ID */			    \
									    \
	static const char* TypeName;					    \
									    \
	static typeG*const Nil;   /* The Nil value for this Type */	    \
									    \
	/* Dynamic Type Determination */				    \
									    \
	    inline virtual int						    \
	isType (void* TestType) const					    \
	   {return (TestType == Type) || parentG::isType(TestType);}	    \
									    \
        /* Constructors and Operations */				    \


#define CLASS_G(type, parent)						    \
CLASS_GGA(CAT(type,G), CAT(parent,G), CAT(type,A))			    \


#define CLASS_SUB_G(subclass, type)					    \
CLASS_GGA(CAT(subclass,G), CAT(type,G), CAT(type,A))			    \


/////////////////////////////////////////////////////////////////////////////
// Inline Functions

#define INLINES_AAG(typeA, ParentA, typeG)		    		    \
									    \
/* Constructors */							    \
    inline typeA::							    \
typeA ()								    \
   :ParentA()								    \
   {}									    \
                                                                            \
    inline typeA::							    \
typeA (const typeG* iGuts)						    \
   :ParentA(iGuts)							    \
   {}									    \
                                                                            \
    inline typeA::							    \
typeA (const typeA& O)							    \
   :ParentA(O)								    \
   {}									    \
                                                                            \
/* Operations */							    \
    inline typeA& typeA::						    \
operator = (const typeA& O)						    \
   {return (typeA&)(oathCoreA::operator=(O));}				    \
                                                                            \
    inline typeG * typeA::						    \
guts () const								    \
   {return (typeG*)(oathCoreA::guts());}				    \
									    \
/* Static Operations */							    \
    inline const typeA& typeA::						    \
isa (const oathCoreA& O)						    \
   {if(O.isType(typeG::Type))						    \
	return (const typeA&)O;						    \
    else								    \
        return Nil;							    \
   }									    \
                                                                            \
    inline const typeA& typeA::						    \
isa (const typeA& O)							    \
   {return O;}								    \


#define INLINES(type, parent)						    \
INLINES_AAG(CAT(type,A), CAT(parent,A), CAT(type,G))			    \


/////////////////////////////////////////////////////////////////////////////
// Outline Functions and Static Member Initialization

#define OUTLINES_G(type, parent)					    \
    void CAT(type,G)::							    \
vtablePosition () const							    \
   {}									    \
									    \
    void* CAT(type,G)::							    \
Type     = &CAT(type,G)::Type;						    \
									    \
    CAT(type,G)*const CAT(type,G)::					    \
Nil      = (CAT(type,G)*)(CAT(type,G)::nil());				    \
									    \
    const char* CAT(type,G)::						    \
TypeName = oathCoreG::registerType(STRING(type), CAT(type,G)::import);	    \


#define OUTLINES(type, parent)						    \
OUTLINES_G(type, parent)						    \
									    \
    const CAT(type,A) CAT(type,A)::					    \
Nil      = CAT(type,A)(CAT(type,G)::Nil);				    \


//***************************************************************************

#endif /* OATH_MACROS_H */
