#ifndef OATH_MINSTRING_H

#define OATH_MINSTRING_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  minString (minStringA, minStringG, dlMinStringG, pMinStringG)
//  minStringPos (minStringPosA, minStringPosG, dlMinStringPosG, pMinStringPosG)
//
//  History:
//    07/91  Brian M Kennedy  import, export, typeRegister
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include <string.h>

#include "string.h"

#include "dlcNodeP.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

class minStringPosA;
class minStringPosG;

class pMinStringPosG;
class dlMinStringPosG;


/////////////////////////////////////////////////////////////////////////////
// class minStringA

CLASS_A(minString, string)

	// Makes //////////

            static inline minStringA
        make (int MakeConst = FALSE);

            static inline minStringA
        make (const seqA& S, int MakeConst = FALSE);

            static inline minStringA
        make (const posA& S, const posA& B, int MakeConst = FALSE);

            static inline minStringA
        make (const char* S, int MakeConst = FALSE);

            static inline minStringA
        make (const char* S, const char* B, int MakeConst = FALSE);


            static inline minStringA
        make (int Uppercase, const seqA& S, int MakeConst = FALSE);

            static inline minStringA
        make (int Uppercase, const posA& S, const posA& B, int MakeConst=FALSE);

            static inline minStringA
        make (int Uppercase, const char* S, int MakeConst = FALSE);

            static inline minStringA
        make (int Uppercase, const char* S, const char* B, int MakeConst=FALSE);


            static inline minStringA
        make (istream&, int MakeConst = FALSE);


	// Operations //////////

	    inline minStringPosA
	makePos (int I = 0, int MakeConst = FALSE) const;

	string_RETURNING_DEFINITIONS(minString)

   };


/////////////////////////////////////////////////////////////////////////////
// class minStringG

CLASS_G(minString, string)

    friend class minStringPosG;

	// Constructors //////////

	    inline
        minStringG (int IsConst = FALSE);

    public:
	// oathCore Operations //////////

	    inline virtual int
        isImplementedAs (void* TypeId) const;

   };


/////////////////////////////////////////////////////////////////////////////
// class dlMinStringG

CLASS_SUB_G(dlMinString, minString)

    friend class pMinStringG;
    friend class dlMinStringPosG;

	// Members //////////

    	dlcNodeP * Header;
    	dlcNodeP * End;
	dlMinStringPosG * PosList;

	// Constructors //////////

        dlMinStringG (int IsConst = FALSE);

	    inline
        dlMinStringG (dlcNodeP* iHeader, dlcNodeP* iEnd, int IsConst = FALSE);

        dlMinStringG (const seqG* S, int IsConst = FALSE);

        dlMinStringG (const posG* Start, const posG* Beyond, int IsConst=FALSE);

        dlMinStringG (const char* S, int IsConst = FALSE);

        dlMinStringG (const char* Start, const char* Beyond, int IsConst=FALSE);

        dlMinStringG (int Uppercase, const seqG* S, int IsConst = FALSE);

        dlMinStringG (int Uppercase, 
		      const posG* Start, const posG* Beyond, int IsConst=FALSE);

        dlMinStringG (int Uppercase, const char* S, int IsConst = FALSE);

        dlMinStringG (int Uppercase, 
		      const char* Start, const char* Beyond, int IsConst=FALSE);

        ~dlMinStringG ();

    public:
        // oathCore Operations //////////

            inline virtual int
        isImplementedAs (void* TypeId) const;

	    virtual void
	export (exportP& X) const;

	    static objA
	import (importP& M);


        // obj Operations //////////

            virtual int
        isEqual (const objG* O) const;

            inline virtual objA
        makeCopy (int MakeConst) const;


        // bag Operations //////////

            virtual int
        isEmpty () const;

            virtual int
        count () const;

            virtual int
        contains (const objG* O) const;

            virtual int
        containsEqual (const objG* O) const;

            virtual int
        canContain (const objG* O) const;

            virtual void
        insert (const objG* O);

            virtual void
        append (const bagG* B);

            virtual void
        apply (void (*F)(objA)) const;

            virtual bagG*
        applyX (objA (*F)(objA), bagG* B) const;

            virtual bagG*
        applyX (bagA (*F)(objA), bagG* B) const;

            inline virtual bagA
        makeEmpty () const;


        // queue Operations //////////

	    inline virtual const objG*
	front () const;

	    virtual const objG*
	remove ();


        // seq Operations //////////

            virtual const objG*
        subscript (int I) const;

            virtual const objG*
        subscript (const posG* P) const;

            inline virtual posA
        makePos (int I, int MakeConst) const;

	    virtual seqA
        makeSeq (const posG* Start, const posG* Beyond, int MakeConst) const;

	    virtual seqA
        makeSeq (int Start, int Beyond, int MakeConst) const;


        // fifoQueue Operations //////////


        // deq Operations //////////

            inline virtual const objG*
        end () const;

            virtual void
        pushFront (const objG* O);

            virtual const objG*
        pullEnd ();


        // list Operations //////////

            virtual void
        insertList (const listPosG*, const objG*);

            virtual const objG*
	removeList (const listPosG*);


        // string Operations //////////

	    virtual int
	isEqual (const char * C) const;

	    virtual int
	isEqual (const stringPosG*, const stringPosG*) const;

	    virtual int
	isEqualAnyCase (const stringG* S) const;

	    virtual int
	isEqualAnyCase (const char * C) const;

	    virtual int
	isEqualAnyCase (const stringPosG*, const stringPosG*) const;

	    virtual int
        lessThan (const stringG* S) const;

	    virtual int
        moreThan (const stringG* S) const;

            virtual void
        stringInsert (const characterG*);

	    virtual void
        stringInsert (char);

	    virtual void
        stringAppend (const stringG*);

	    virtual void
        stringAppend (const char*);

	    virtual const char *
	charStar () const;

	    virtual void
	charStarX (char * C, int N) const;

	    virtual int
	hash () const;

	    virtual int
	hash (int K) const;

	    virtual int
	hashLower () const;

	    virtual int
	hashLower (int K) const;

	    virtual int
	hashUpper () const;

	    virtual int
	hashUpper (int K) const;

            virtual stringG*
        applyStringX (characterA (*F)(characterA), stringG* S) const;

            virtual stringG*
        applyStringX (stringA (*F)(characterA), stringG* S) const;

	    virtual void
	lowercase ();

	    virtual void
	uppercase ();

	    virtual stringA
	makeLower (int MakeConst) const;

	    virtual stringA
	makeUpper (int MakeConst) const;

	    virtual ostream&
	ostreamInsertion (ostream& F) const;

	    virtual istream&
	istreamExtraction (istream& F);


        // dlMinString Operations //////////

            dlcNodeP *
        internalPosition (int I) const;

	    inline dlMinStringPosG*&
	posList () const;	// not really const!!!

	    void
	adjustPosList (dlcNodeP * CurrentPrev, dlcNodeP * NewPrev);

	    inline void
	adjustEnd (dlcNodeP * OldEnd, dlcNodeP * NewEnd) const; // Not really!!

   };


/////////////////////////////////////////////////////////////////////////////
// class pMinStringG

CLASS_SUB_G(pMinString, minString)

    friend class dlMinStringG;
    friend class pMinStringPosG;

        // Members //////////

        int    Count;	// Number of char's (excluding ending \0) 
	char * String;

	// Constructors //////////

        pMinStringG ();

	    inline
        pMinStringG (char * iString, int iCount);  // iString NOT copied!!

        pMinStringG (const seqG* S);

        pMinStringG (const posG* Start, const posG* Beyond);

        pMinStringG (const char * S);		// NUL-terminated string

        pMinStringG (const char * Start, const char * Beyond);

        pMinStringG (int Uppercase, const seqG* S);

        pMinStringG (int Uppercase, const posG* Start, const posG* Beyond);

        pMinStringG (int Uppercase, const char * S);

        pMinStringG (int Uppercase, const char * Start, const char * Beyond);

	    inline
        ~pMinStringG ();


    public:
	// oathCore Operations //////////

	    inline virtual int
        isImplementedAs (void* TypeId) const;

	    virtual void
	export (exportP& X) const;

	    static objA
	import (importP& M);


	// obj Operations //////////

            virtual int
        isEqual (const objG* O) const;

            inline virtual objA
        makeCopy (int MakeConst) const;


        // bag Operations //////////

            virtual int
        isEmpty () const;

            virtual int
        count () const;

            virtual int
        contains (const objG* O) const;

            virtual int
        containsEqual (const objG* O) const;

            virtual int
        canContain (const objG* O) const;

            virtual void
        insert (const objG*);

            virtual void
        append (const bagG*);

            virtual void
        apply (void (*F)(objA)) const;

            virtual bagG*
        applyX (objA (*F)(objA), bagG* B) const;

            virtual bagG*
        applyX (bagA (*F)(objA), bagG* B) const;

            inline virtual bagA
        makeEmpty () const;


        // queue Operations //////////

	    inline virtual const objG*
	front () const;

	    virtual const objG*
	remove ();


        // seq Operations //////////

            virtual const objG*
        subscript (int I) const;

            virtual const objG*
        subscript (const posG* P) const;

            inline virtual posA
        makePos (int I, int MakeConst) const;

	    virtual seqA
        makeSeq (const posG* Start, const posG* Beyond, int MakeConst) const;

	    virtual seqA
        makeSeq (int Start, int Beyond, int MakeConst) const;


        // fifoQueue Operations //////////


        // deq Operations //////////

            inline virtual const objG*
        end () const;

            virtual void
        pushFront (const objG* O);

            virtual const objG*
        pullEnd ();


        // list Operations //////////

            virtual void
        insertList (const listPosG*, const objG*);

            virtual const objG*
	removeList (const listPosG*);


        // string Operations //////////

	    virtual int
	isEqual (const char * C) const;

	    virtual int
	isEqual (const stringPosG*, const stringPosG*) const;

	    virtual int
	isEqualAnyCase (const stringG* S) const;

	    virtual int
	isEqualAnyCase (const char * C) const;

	    virtual int
	isEqualAnyCase (const stringPosG*, const stringPosG*) const;

	    virtual int
        lessThan (const stringG* S) const;

	    virtual int
        moreThan (const stringG* S) const;

            virtual void
        stringInsert (const characterG*);

	    virtual void
        stringInsert (char);

	    virtual void
        stringAppend (const stringG*);

	    virtual void
        stringAppend (const char*);

	    virtual const char *
	charStar () const;

	    virtual void
	charStarX (char * C, int N) const;

	    virtual int
	hash () const;

	    virtual int
	hash (int K) const;

	    virtual int
	hashLower () const;

	    virtual int
	hashLower (int K) const;

	    virtual int
	hashUpper () const;

	    virtual int
	hashUpper (int K) const;

            virtual stringG*
        applyStringX (characterA (*F)(characterA), stringG* S) const;

            virtual stringG*
        applyStringX (stringA (*F)(characterA), stringG* S) const;

	    virtual void
	lowercase ();

	    virtual void
	uppercase ();

	    virtual stringA
	makeLower (int MakeConst) const;

	    virtual stringA
	makeUpper (int MakeConst) const;

	    virtual ostream&
	ostreamInsertion (ostream& F) const;

	    virtual istream&
	istreamExtraction (istream& F);


	// pMinString Operations //////////

	    inline const char *
	internalPastEnd () const;

	    inline const char *
        internalPosition (int I) const;

   };


/////////////////////////////////////////////////////////////////////////////
// class minStringPosA

CLASS_A(minStringPos, stringPos)

	stringPos_RETURNING_DEFINITIONS(minStringPos)

   };


/////////////////////////////////////////////////////////////////////////////
// class minStringPosG

CLASS_G(minStringPos, stringPos)

    friend class minStringG;

	// Members //////////

        const minStringA String;

	// Constructors //////////

	    inline
        minStringPosG (int IsConst = FALSE);

	    inline
        minStringPosG (const minStringG* Parent, int IsConst = FALSE);

    public:
        // oathCore Operations //////////

            inline virtual int
        isImplementedAs (void* TypeId) const;


        // pos Operations //////////

            inline virtual const seqG*
        parent () const;

   };


/////////////////////////////////////////////////////////////////////////////
// class dlMinStringPosG

CLASS_SUB_G(dlMinStringPos, minStringPos)

    friend class dlMinStringG;

	// Members //////////

        dlcNodeP *         Prev;
	dlMinStringPosG *  PrevPos;
	dlMinStringPosG *  NextPos;

	// Constructors //////////

	    inline
        dlMinStringPosG (int IsConst = FALSE);

	dlMinStringPosG (const minStringG* iString, dlcNodeP* iPrev, 
						    int IsConst = FALSE);

	~dlMinStringPosG ();

    public:
        // oathCore Operations //////////

	    inline virtual int
	isImplementedAs (void* TypeId) const;

	    virtual void
	export (exportP& X) const;

	    static objA
	import (importP& M);


	// obj Operations //////////

            virtual int
        isEqual (const objG* O) const;

            inline virtual objA
        makeCopy (int MakeConst) const;


        // pos Operations //////////

            inline virtual int
        isPastEnd () const;

            virtual const objG*
        indirection () const;

            virtual void
        increment ();

            virtual void
        increment (int I);

            virtual const objG*
        find (const objG* O);

            virtual const objG*
        findEqual (const objG* O);

            virtual void
        reset (const posG* P);

            virtual void
        reset (int I);


        // listPos Operations //////////

            virtual void
        decrement ();

            virtual void
        decrement (int I);


        // stringPos Operations //////////

            virtual const characterG*
        charIndirection () const;

	    virtual int 
	skipSpaces ();

	    virtual const stringG*
	match (const stringG* S);

	    virtual int
	canMatch (const stringG* S) const;

	    virtual const stringG*
	findString (const stringG* S);

	    virtual const stringG*
	findMatch (const stringG* S);

	    virtual const stringG*
	matchAnyCase (const stringG* S);

	    virtual int
	canMatchAnyCase (const stringG* S) const;

	    virtual const stringG*
	findAnyCase (const stringG* S);

	    virtual const stringG*
	findMatchAnyCase (const stringG* S);

	    virtual const char *
	match (const char * S);

	    virtual int
	canMatch (const char * S) const;

	    virtual const char *
	findString (const char * S);

	    virtual const char *
	findMatch (const char * S);

	    virtual const char *
	matchAnyCase (const char * S);

	    virtual int
	canMatchAnyCase (const char * S) const;

	    virtual const char *
	findAnyCase (const char * S);

	    virtual const char *
	findMatchAnyCase (const char * S);


        // dlMinStringPos Operations //////////

	    inline const dlMinStringG*
	parentT () const;	// not really const!!!

	    inline dlcNodeP*&
	prev () const; // Not really const!!

	    inline virtual void
	adjustEnd (dlcNodeP * NewEnd) const; // Not really const!!

   };


/////////////////////////////////////////////////////////////////////////////
// class pMinStringPosG

CLASS_SUB_G(pMinStringPos, minStringPos)

    friend class pMinStringG;

	// Members //////////

        const char * Char;

	// Constructors //////////

	    inline
        pMinStringPosG (int IsConst = FALSE);

	    inline
        pMinStringPosG (const minStringG* iString, const char * iChar,
		        int IsConst = FALSE);


    public:
        // oathCore Operations //////////

	    inline virtual int
	isImplementedAs (void* TypeId) const;

	    virtual void
	export (exportP& X) const;

	    static objA
	import (importP& M);


	// obj Operations //////////

            virtual int
        isEqual (const objG* O) const;

            inline virtual objA
        makeCopy (int MakeConst) const;


        // pos Operations //////////

            inline virtual int
        isPastEnd () const;

            virtual const objG*
        indirection () const;

            virtual void
        increment ();

            virtual void
        increment (int I);

            virtual const objG*
        find (const objG* O);

            virtual const objG*
        findEqual (const objG* O);

            virtual void
        reset (const posG* P);

            virtual void
        reset (int I);


        // listPos Operations //////////

            virtual void
        decrement ();

            virtual void
        decrement (int I);


        // stringPos Operations //////////

            virtual const characterG*
        charIndirection () const;

	    virtual int 
	skipSpaces ();

	    virtual const stringG*
	match (const stringG* S);

	    virtual int
	canMatch (const stringG* S) const;

	    virtual const stringG*
	findString (const stringG* S);

	    virtual const stringG*
	findMatch (const stringG* S);

	    virtual const stringG*
	matchAnyCase (const stringG* S);

	    virtual int
	canMatchAnyCase (const stringG* S) const;

	    virtual const stringG*
	findAnyCase (const stringG* S);

	    virtual const stringG*
	findMatchAnyCase (const stringG* S);

	    virtual const char *
	match (const char * S);

	    virtual int
	canMatch (const char * S) const;

	    virtual const char *
	findString (const char * S);

	    virtual const char *
	findMatch (const char * S);

	    virtual const char *
	matchAnyCase (const char * S);

	    virtual int
	canMatchAnyCase (const char * S) const;

	    virtual const char *
	findAnyCase (const char * S);

	    virtual const char *
	findMatchAnyCase (const char * S);


	// pMinStringPos Operations //////////

	    inline const pMinStringG*
	parentT () const;	// not really const!!!

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// minStringG Inlines

// Constructors //////////

    inline minStringG::
minStringG (int IsConst)
   :stringG(IsConst)
   {}

// oathCore Operations //////////

    inline int minStringG::
isImplementedAs (void* TypeId) const
   {return TypeId == Type;}


/////////////////////////////////////////////////////////////////////////////
// dlMinStringG Inlines

// Constructors //////////

    inline dlMinStringG::
dlMinStringG (dlcNodeP* iHeader, dlcNodeP* iEnd, int IsConst)
   :minStringG(IsConst), Header(iHeader), End(iEnd), PosList(0)
   {}


// oathCore Operations //////////

    inline int dlMinStringG::
isImplementedAs (void* TypeId) const        // Not really const!!
   {return TypeId == Type || minStringG::isImplementedAs(TypeId);}


// bag Operations //////////

    inline bagA dlMinStringG::
makeEmpty () const
   {return new dlMinStringG ();}


// queue Operations //////////

    inline const objG* dlMinStringG::
front () const
   {assumed(!isEmpty(), "top() attempted on an empty Stack!");
    return Header->nextObj().guts();
   }


// seq Operations //////////

    inline posA dlMinStringG::
makePos (int I, int MakeConst) const
   {return new dlMinStringPosG (this, internalPosition(I), MakeConst);}


// fifoQueue Operations //////////

// deq Operations //////////

    inline const objG* dlMinStringG::
end () const
   {assumed(!isEmpty(), "end() attempted on an empty String!");
    return End->thisObj().guts();
   }


// list Operations //////////

// string Operations //////////

// dlMinString Operations //////////

    inline dlMinStringPosG*& dlMinStringG::
posList () const	// not really const!!!
   {return ((dlMinStringG*)this)->PosList;}

    inline void dlMinStringG::
adjustEnd (dlcNodeP * OldEnd, dlcNodeP * NewEnd) const // Not really!!
   {if(End == OldEnd)
	((dlMinStringG*)this)->End = NewEnd;
   }


/////////////////////////////////////////////////////////////////////////////
// pMinStringG Inlines

// Constructors //////////

    inline pMinStringG::
pMinStringG (char * iString, int iCount)
   :minStringG(TRUE), Count(iCount), String(iString)
   {}

    inline pMinStringG::
~pMinStringG ()
   {delete [] String;}

// Inner operations

    inline const char * pMinStringG::
internalPastEnd () const
   {return String + Count;}

    inline const char * pMinStringG::
internalPosition (int I) const
   {I = I > Count ? Count : I;
    return String + I;
   }


// oathCore Operations //////////

    inline int pMinStringG::
isImplementedAs (void* TypeId) const
   {return TypeId == Type || minStringG::isImplementedAs(TypeId);}


// obj Operations //////////

    inline objA pMinStringG::
makeCopy (int MakeConst) const
   {return MakeConst ? (objG*)this 
		     : (objG*)new dlMinStringG (String, String + Count);}


// bag Operations //////////

    inline bagA pMinStringG::
makeEmpty () const
   {return new dlMinStringG ();}

// queue Operations //////////

    inline const objG* pMinStringG::
front () const
   {ensure(Count, "operator * attempted on an empty string!");
    return characterA::make(String[0]).guts();
   }

// seq Operations //////////

    inline posA pMinStringG::
makePos (int I, int MakeConst) const
   {return new pMinStringPosG (this, internalPosition(I), MakeConst);}


// fifoQueue Operations //////////

// deq Operations //////////

    inline const objG* pMinStringG::
end () const
   {ensure(Count, "operator * attempted on an empty string!");
    return characterA::make(String[Count-1]).guts();
   }


// list Operations //////////

// string Operations //////////

// pMinString Operations //////////


/////////////////////////////////////////////////////////////////////////////
// minStringA Inlines

INLINES(minString, string)

    inline minStringA minStringA::
make (int MakeConst)
   {return MakeConst ? (minStringG*)new pMinStringG()
		     : (minStringG*)new dlMinStringG();
   }

    inline minStringA minStringA::
make (const seqA& S, int MakeConst)
   {assumed(!!S, "Attempted to make a pdString from a Nil!");
    return MakeConst ? (minStringG*)new pMinStringG(S.guts())
		     : (minStringG*)new dlMinStringG(S.guts());
   }

    inline minStringA minStringA::
make (const posA& S, const posA& B, int MakeConst)
   {assumed(!!S, "Attempted to make a pdString from a Nil!");
    return MakeConst ? (minStringG*)new pMinStringG(S.guts(), B.guts())
		     : (minStringG*)new dlMinStringG(S.guts(), B.guts());
   }

    inline minStringA minStringA::
make (const char* S, int MakeConst)
   {return MakeConst ? (minStringG*)new pMinStringG(S)
		     : (minStringG*)new dlMinStringG(S);
   }

    inline minStringA minStringA::
make (const char* S, const char* B, int MakeConst)
   {return MakeConst ? (minStringG*)new pMinStringG (S, B)
		     : (minStringG*)new dlMinStringG(S, B);}

    inline minStringA minStringA::
make (int Uppercase, const seqA& S, int MakeConst)
   {assumed(!!S, "Attempted to make a pdString from a Nil!");
    return MakeConst ? (minStringG*)new pMinStringG (Uppercase, S.guts())
		     : (minStringG*)new dlMinStringG(Uppercase, S.guts());
   }

    inline minStringA minStringA::
make (int Uppercase, const posA& S, const posA& B, int MakeConst)
   {assumed(!!S, "Attempted to make a pdString from a Nil!");
    return MakeConst
	   ? (minStringG*)new pMinStringG (Uppercase, S.guts(), B.guts())
	   : (minStringG*)new dlMinStringG(Uppercase, S.guts(), B.guts());
   }

    inline minStringA minStringA::
make (int Uppercase, const char* S, int MakeConst)
   {return MakeConst ? (minStringG*)new pMinStringG (Uppercase, S)
		     : (minStringG*)new dlMinStringG(Uppercase, S);
   }

    inline minStringA minStringA::
make (int Uppercase, const char* S, const char* B, int MakeConst)
   {return MakeConst ? (minStringG*)new pMinStringG (Uppercase, S, B)
		     : (minStringG*)new dlMinStringG(Uppercase, S, B);
   }


    inline minStringA minStringA::
make (istream& F, int MakeConst)
   {minStringA M = make();
    F >> M;
    if(MakeConst)
        return make(M, TRUE);
    else
        return M;
   }


    inline minStringPosA minStringA::
makePos (int I, int MakeConst) const
   {return (minStringPosA&)am().guts()->makePos(I, MakeConst);}


string_RETURNING_INLINES(minString)


/////////////////////////////////////////////////////////////////////////////
// minStringPosG Inlines

// Constructors //////////

    inline minStringPosG::
minStringPosG (int IsConst)
   :stringPosG(IsConst), String(minStringG::Nil)
   {}

    inline minStringPosG::
minStringPosG (const minStringG* iString, int IsConst)
   :stringPosG(IsConst), String(iString)
   {}


// oathCore Operations //////////

    inline int minStringPosG::
isImplementedAs (void* TypeId) const
   {return TypeId == Type;}

// pos Operations //////////

    inline const seqG* minStringPosG::
parent () const
   {return (const seqG*)String.guts();}


/////////////////////////////////////////////////////////////////////////////
// dlMinStringPosG Inlines

    inline const dlMinStringG* dlMinStringPosG::
parentT () const
   {return (const dlMinStringG*)String.guts();}

// Constructors //////////

    inline dlMinStringPosG::
dlMinStringPosG (int IsConst)
   :minStringPosG(IsConst)
   {}


// oathCore Operations //////////

    inline int dlMinStringPosG::
isImplementedAs (void* TypeId) const
   {return TypeId == Type || minStringPosG::isImplementedAs(TypeId);}


// obj Operations //////////

    inline objA dlMinStringPosG::
makeCopy (int MakeConst) const
   {return new dlMinStringPosG(String.guts(), Prev, MakeConst);}


// pos Operations //////////

    inline int dlMinStringPosG::
isPastEnd () const
   {return !Prev->next();}

// listPos Operations //////////

// stringPos Operations //////////

// dlMinStringPos Operations //////////

    inline dlcNodeP*& dlMinStringPosG::
prev () const // Not really const!!
   {return ((dlMinStringPosG*)this)->Prev;}

    inline void dlMinStringPosG::
adjustEnd (dlcNodeP * NewEnd) const // Not really const!!
   {parentT()->adjustEnd(Prev, NewEnd);
    ((dlMinStringPosG*)this)->Prev = NewEnd;
   }


/////////////////////////////////////////////////////////////////////////////
// pMinStringPosG Inlines

    inline const pMinStringG* pMinStringPosG::
parentT () const
   {return (const pMinStringG*)String.guts();}

// Constructors //////////

    inline pMinStringPosG::
pMinStringPosG (int IsConst)
   :minStringPosG(IsConst), Char(0)
   {}

    inline pMinStringPosG::
pMinStringPosG (const minStringG* iString, const char * iChar, int IsConst)
   :minStringPosG(iString, IsConst), Char(iChar)
   {}


// oathCore Operations //////////

    inline int pMinStringPosG::
isImplementedAs (void* TypeId) const
   {return TypeId == Type || minStringPosG::isImplementedAs(TypeId);}


// obj Operations //////////

    inline objA pMinStringPosG:: 
makeCopy (int MakeConst) const
   {return new pMinStringPosG(String.guts(), Char, MakeConst);}


// pos Operations //////////

    inline int pMinStringPosG::
isPastEnd () const
   {return Char == parentT()->internalPastEnd();}


// listPos Operations //////////

// stringPos Operations //////////

// pMinStringPos Operations //////////


/////////////////////////////////////////////////////////////////////////////
// minStringPosA Inlines

INLINES(minStringPos, stringPos)

stringPos_RETURNING_INLINES(minStringPos)


//***************************************************************************

#endif /* OATH_MINSTRING_H */

