#ifndef OATH_NUMERIC_H

#define OATH_NUMERIC_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  complex (complexA, complexG)
//  real (realA, realG)
//  rational (rationalA, rationalG)
//  integer (integerA, integerG)
//
//  History:
//    08/91  Brian M Kennedy  signMagnitudeP; functionality refined
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    02/91  Brian M Kennedy  Original
//
//***************************************************************************

#include "obj.h"

class signMagnitudeP;

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Declarations

class complexA;
class realA;
class rationalA;
class integerA;

class complexG;
class realG;
class rationalG;
class integerG;


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class complexA

CLASS_A(complex, obj)

	// Operations

            inline realA
        makeReal (int MakeConst = FALSE) const;

            inline realA
        makeImag (int MakeConst = FALSE) const;

	    inline realA
	makeAbs (int MakeConst = FALSE) const;

	    inline realA       // in radians
	makeArg (int MakeConst = FALSE) const;

	    inline int
	hasZero () const;

	    inline int
	hasZeroImag () const;

	    inline int
	hasEqual (const complexA&) const;

	    inline void
	print (ostream& F, char Sep = '\0') const;


#define complex_RETURNING_DEFINITIONS(type)				    \
	obj_RETURNING_DEFINITIONS(type)					    \
									    \
	    inline const CAT(type,A)&					    \
	reset () const;							    \
									    \
	    inline const CAT(type,A)&					    \
	reset (const complexA& C) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator - () const;						    \
									    \
	    inline const CAT(type,A)&					    \
	operator ++ () const;						    \
									    \
	    inline const CAT(type,A)&					    \
	operator += (const complexA& C) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator + (const complexA& C) const;				    \
									    \
	    inline const CAT(type,A)&					    \
	operator -- () const;						    \
									    \
	    inline const CAT(type,A)&					    \
	operator -= (const complexA& C) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator - (const complexA& C) const;				    \
									    \
	    inline const CAT(type,A)&					    \
	operator *= (const complexA& C) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator * (const complexA& C) const;				    \
									    \
	    inline const CAT(type,A)&					    \
	operator /= (const complexA& C) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator / (const complexA& C) const;				    \
									    \
	    inline const CAT(type,A)&					    \
	operator <<= (const integerA& Count) const;			    \
									    \
	    inline CAT(type,A)						    \
	operator << (const integerA& Count) const;			    \
									    \
	    inline const CAT(type,A)&					    \
	operator >>= (const integerA& Count) const;			    \
									    \
	    inline CAT(type,A)						    \
	operator >> (const integerA& Count) const;			    \
									    \
	    inline const CAT(type,A)&					    \
	neg () const;							    \
									    \
	    inline const CAT(type,A)&					    \
	conj () const;							    \
									    \
	    inline const CAT(type,A)&					    \
	abs () const;							    \
									    \
	    inline const CAT(type,A)&					    \
	inv () const;							    \
									    \
	    inline const CAT(type,A)&					    \
	sqr () const;							    \
									    \
	    inline const CAT(type,A)&					    \
	sqrt () const;							    \
									    \
	    inline const CAT(type,A)&					    \
	pow (const integerA& Expn) const;				    \


	complex_RETURNING_DEFINITIONS(complex)

   };


/////////////////////////////////////////////////////////////////////////////
// class complexG

CLASS_G(complex, obj)

	// Constructors //////////

	    inline
        complexG (int IsConst = FALSE);

    public:
        // complex Operations //////////

	    virtual void
	reset () = 0;

	    virtual void
	reset (const complexG* C) = 0;

	    virtual realA
	makeReal (int MakeConst) const = 0;

	    virtual realA
	makeImag (int MakeConst) const = 0;

	    virtual realA
	makeAbs (int MakeConst) const = 0;

	    virtual realA
	makeArg (int MakeConst) const = 0;

	    virtual int
	hasZero () const = 0;

	    virtual int
	hasZeroImag () const = 0;

	    virtual int
	hasEqual (const complexG* C) const = 0;

	    virtual void
	increment () = 0;

	    virtual void
	increment (const complexG* C) = 0;

	    virtual void
	decrement () = 0;

	    virtual void
	decrement (const complexG* C) = 0;

	    virtual void
	multiply (const complexG* C) = 0;

	    virtual void
	divide (const complexG* C) = 0;

	    virtual void
	shiftLeft (const integerG* Count) = 0;

	    virtual void
	shiftRight (const integerG* Count) = 0;

	    virtual void
	neg () = 0;

	    virtual void
	conj () = 0;

	    virtual void
	abs () = 0;

	    virtual void
	inv () = 0;

	    virtual void
	sqr () = 0;

	    virtual void
	sqrt () = 0;

	    virtual void
	pow (const integerG* Expn) = 0;

	    virtual void
	print (ostream& F, char Sep) const = 0;

   };


/////////////////////////////////////////////////////////////////////////////
// class realA

CLASS_A(real, complex)

	// Implementors' Operations

	    inline const signMagnitudeP&
	smMantissa () const;

	    inline const signMagnitudeP&
	smExponent () const;

	// Operations

            inline integerA
        makeFloor (int MakeConst = FALSE) const;

            inline integerA
        makeCeiling (int MakeConst = FALSE) const;

            inline integerA
        makeRound (int MakeConst = FALSE) const;

            inline integerA
        makeTruncate (int MakeConst = FALSE) const;

	    inline int
	sign () const;

	    inline int
	hasPositive () const;

	    inline int
	hasNegative () const;

	    inline int
	operator < (const realA&) const;

	    inline int
	operator <= (const realA&) const;

	    inline int
	operator >= (const realA&) const;

	    inline int
	operator > (const realA&) const;

	    inline int
	hasDouble () const;

	    inline double
	makeDouble () const;


#define real_RETURNING_DEFINITIONS(type)				    \
	complex_RETURNING_DEFINITIONS(type)				    \
									    \
	    inline const CAT(type,A)&					    \
	min (const realA& R) const;					    \
									    \
	    inline const CAT(type,A)&					    \
	max (const realA& R) const;					    \


	real_RETURNING_DEFINITIONS(real)


   };


/////////////////////////////////////////////////////////////////////////////
// class realG

CLASS_G(real, complex)

	// Constructors //////////

	    inline
        realG (int IsConst = FALSE);

    public:
        // real Operations //////////

	    virtual const signMagnitudeP&
	smMantissa () const = 0;

	    virtual const signMagnitudeP&
	smExponent () const = 0;

	    virtual integerA
	makeFloor (int MakeConst) const = 0;

	    virtual integerA
	makeCeiling (int MakeConst) const = 0;

	    virtual integerA
	makeRound (int MakeConst) const = 0;

	    virtual integerA
	makeTruncate (int MakeConst) const = 0;

	    virtual int
	sign () const = 0;

	    virtual int
	hasPositive () const = 0;

	    virtual int
	hasNegative () const = 0;

	    virtual int
	hasLess (const realG* R) const = 0;

	    virtual int
	hasMore (const realG* R) const = 0;

	    virtual int
	hasDouble () const = 0;

	    virtual double
	makeDouble () const = 0;

	    virtual void
	min (const realG* R) = 0;

	    virtual void
	max (const realG* R) = 0;

   };


/////////////////////////////////////////////////////////////////////////////
// class rationalA

CLASS_A(rational, real)

	// Operations

            inline integerA
        makeNumerator (int MakeConst = FALSE) const;

            inline integerA
        makeDenominator (int MakeConst = FALSE) const;


#define rational_RETURNING_DEFINITIONS(type)				    \
	real_RETURNING_DEFINITIONS(type)				    \


	rational_RETURNING_DEFINITIONS(rational)

   };


/////////////////////////////////////////////////////////////////////////////
// class rationalG

CLASS_G(rational, real)

	// Constructors //////////

	    inline
        rationalG (int IsConst = FALSE);

    public:
        // rational Operations //////////

	    virtual integerA
	makeNumerator (int MakeConst) const = 0;

	    virtual integerA
	makeDenominator (int MakeConst) const = 0;

   };


/////////////////////////////////////////////////////////////////////////////
// class integerA

CLASS_A(integer, rational)

	// Operations

	    inline int
	hasEven () const;

	    inline int
	hasOdd () const;

	    inline int
	hasLong () const;

	    inline long
	makeLong () const;


#define integer_RETURNING_DEFINITIONS(type)				    \
	rational_RETURNING_DEFINITIONS(type)				    \
									    \
	    inline const CAT(type,A)&					    \
	operator %= (const integerA& I) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator % (const integerA& I) const;				    \
									    \
	    inline const CAT(type,A)&					    \
	remDivX (const integerA& Divisor, const integerA& Quot) const;	    \
									    \
	    inline const CAT(type,A)&					    \
	gcd (const integerA& I) const;					    \
									    \
	    inline const CAT(type,A)&					    \
	lcm (const integerA& I) const;					    \


	integer_RETURNING_DEFINITIONS(integer)

   };


/////////////////////////////////////////////////////////////////////////////
// class integerG

CLASS_G(integer, rational)

	// Constructors //////////

	    inline
        integerG (int IsConst = FALSE);

    public:
        // integer Operations //////////

	    virtual int
	hasEven () const = 0;

	    virtual int
	hasLong () const = 0;

	    virtual long
	makeLong () const = 0;

	    virtual void
	rem (const integerG*) = 0;

	    virtual void
	remDivX (const integerG*, integerG*) = 0;

	    virtual void
	gcd (const integerG*) = 0;

	    virtual void
	lcm (const integerG*) = 0;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// complexG Inlines

// Constructors //////////

    inline complexG::
complexG (int IsConst) // IsConst = FALSE
   :objG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// complexA Inlines

INLINES(complex, obj)

// Operations

    inline realA complexA::
makeReal (int MakeConst) const
   {return am().guts()->makeReal(MakeConst);}

    inline realA complexA::
makeImag (int MakeConst) const
   {return am().guts()->makeImag(MakeConst);}

    inline realA complexA::
makeAbs (int MakeConst) const
   {return am().guts()->makeAbs(MakeConst);}

    inline realA complexA::
makeArg (int MakeConst) const
   {return am().guts()->makeArg(MakeConst);}

    inline int complexA::
hasZero () const
   {return am().guts()->hasZero();}

    inline int complexA::
hasZeroImag () const
   {return am().guts()->hasZeroImag();}

    inline int complexA::
hasEqual (const complexA& C) const
   {return am().guts()->hasEqual(C.am().guts());}

    inline void complexA::
print (ostream& F, char Sep) const
   {am().guts()->print(F, Sep);}


#define complex_RETURNING_INLINES(type)					    \
obj_RETURNING_INLINES(type)						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
reset () const								    \
   {am().guts()->reset();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
reset (const complexA& C) const						    \
   {am().guts()->reset(C.am().guts());					    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator - () const							    \
   {return makeCopy().neg();}						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator ++ () const							    \
   {am().guts()->increment();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator += (const complexA& C) const					    \
   {am().guts()->increment(C.am().guts());				    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator + (const complexA& C) const					    \
   {return makeCopy() += C;}						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator -- () const							    \
   {am().guts()->decrement();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator -= (const complexA& C) const					    \
   {am().guts()->decrement(C.am().guts());				    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator - (const complexA& C) const					    \
   {return makeCopy() -= C;}						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator *= (const complexA& C) const					    \
   {am().guts()->multiply(C.am().guts());				    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator * (const complexA& C) const					    \
   {return makeCopy() *= C;}						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator /= (const complexA& C) const					    \
   {am().guts()->divide(C.am().guts());					    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator / (const complexA& C) const					    \
   {return makeCopy() /= C;}						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator <<= (const integerA& Count) const				    \
   {am().guts()->shiftLeft(Count.am().guts());				    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator << (const integerA& Count) const				    \
   {return makeCopy() <<= Count;}					    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator >>= (const integerA& Count) const				    \
   {am().guts()->shiftRight(Count.am().guts());				    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator >> (const integerA& Count) const				    \
   {return makeCopy() >>= Count;}					    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
neg () const								    \
   {am().guts()->neg();							    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
conj () const								    \
   {am().guts()->conj();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
abs () const								    \
   {am().guts()->abs();							    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
inv () const								    \
   {am().guts()->inv();							    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
sqr () const								    \
   {am().guts()->sqr();							    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
sqrt () const								    \
   {am().guts()->sqrt();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
pow (const integerA& Expn) const					    \
   {am().guts()->pow(Expn.am().guts());					    \
    return *this;							    \
   }									    \


complex_RETURNING_INLINES(complex)


    inline ostream&
operator << (ostream& F, const complexA& C)
   {C.print(F);
    return F;
   }


/////////////////////////////////////////////////////////////////////////////
// realG Inlines

// Constructors //////////

    inline realG::
realG (int IsConst) // IsConst = FALSE
   :complexG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// realA Inlines

INLINES(real, complex)

// Implementors' Operations

    inline const signMagnitudeP& realA::
smMantissa () const
   {return am().guts()->smMantissa();}

    inline const signMagnitudeP& realA::
smExponent () const
   {return am().guts()->smExponent();}

// Operations

    inline integerA realA::
makeFloor (int MakeConst) const
   {return am().guts()->makeFloor(MakeConst);}

    inline integerA realA::
makeCeiling (int MakeConst) const
   {return am().guts()->makeCeiling(MakeConst);}

    inline integerA realA::
makeRound (int MakeConst) const
   {return am().guts()->makeRound(MakeConst);}

    inline integerA realA::
makeTruncate (int MakeConst) const
   {return am().guts()->makeTruncate(MakeConst);}

    inline int realA::
sign () const
   {return am().guts()->sign();}

    inline int realA::
hasPositive () const
   {return am().guts()->hasPositive();}

    inline int realA::
hasNegative () const
   {return am().guts()->hasNegative();}

    inline int realA::
operator < (const realA& R) const
   {return am().guts()->hasLess(R.am().guts());}

    inline int realA::
operator <= (const realA& R) const
   {return !am().guts()->hasMore(R.am().guts());}

    inline int realA::
operator >= (const realA& R) const
   {return !am().guts()->hasLess(R.am().guts());}

    inline int realA::
operator > (const realA& R) const
   {return am().guts()->hasMore(R.am().guts());}

    inline int realA::
hasDouble () const
   {return am().guts()->hasDouble();}

    inline double realA::
makeDouble () const
   {return am().guts()->makeDouble();}


#define real_RETURNING_INLINES(type)					    \
complex_RETURNING_INLINES(type)						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
min (const realA& R) const						    \
   {am().guts()->min(R.am().guts());					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
max (const realA& R) const						    \
   {am().guts()->max(R.am().guts());					    \
    return *this;							    \
   }									    \


real_RETURNING_INLINES(real)


/////////////////////////////////////////////////////////////////////////////
// rationalG Inlines

// Constructors //////////

    inline rationalG::
rationalG (int IsConst) // IsConst = FALSE
   :realG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// rationalA Inlines

INLINES(rational, real)

    inline integerA rationalA::
makeNumerator (int MakeConst) const
   {return am().guts()->makeNumerator(MakeConst);}

    inline integerA rationalA::
makeDenominator (int MakeConst) const
   {return am().guts()->makeDenominator(MakeConst);}


#define rational_RETURNING_INLINES(type)				    \
real_RETURNING_INLINES(type)						    \


rational_RETURNING_INLINES(rational)


/////////////////////////////////////////////////////////////////////////////
// integerG Inlines

// Constructors //////////

    inline integerG::
integerG (int IsConst) // IsConst = FALSE
   :rationalG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// integerA Inlines

INLINES(integer, rational)

// Operations

    inline int integerA::
hasEven () const
   {return am().guts()->hasEven();}

    inline int integerA::
hasOdd () const
   {return !am().guts()->hasEven();}

    inline int integerA::
hasLong () const
   {return am().guts()->hasLong();}

    inline long integerA::
makeLong () const
   {return am().guts()->makeLong();}


#define integer_RETURNING_INLINES(type)					    \
rational_RETURNING_INLINES(type)					    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator %= (const integerA& I) const					    \
   {am().guts()->rem(I.am().guts());					    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator % (const integerA& I) const					    \
   {return makeCopy() %= I;}						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
remDivX (const integerA& Divisor, const integerA& Quot) const		    \
   {am().guts()->remDivX(Divisor.am().guts(), Quot.am().guts());	    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
gcd (const integerA& I) const						    \
   {am().guts()->gcd(I.am().guts());					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
lcm (const integerA& I) const						    \
   {am().guts()->lcm(I.am().guts());					    \
    return *this;							    \
   }									    \


integer_RETURNING_INLINES(integer)


//***************************************************************************

#endif /* OATH_NUMERIC_H */
