#ifndef OATH_OBJ_H

#define OATH_OBJ_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  obj (objA, objG)
//
//  History:
//    07/91  Brian M Kennedy  import, export, typeRegister
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "oathCore.h"

#include "macros.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class objA

CLASS_A(obj, oathCore)

	// Operations //////////

	    inline int
	isConst () const;

	    inline int
	isNil () const;

	    inline int
	operator ! () const;

	    inline int
	is (const objA& O) const;	

	    inline int
	isEqual (const objA& O) const;

	    inline int
	operator == (const objA& O) const;

	    inline int
	operator != (const objA& O) const;


#define self_RETURNING_DEFINITIONS(type)			    	    \
									    \
            inline CAT(type,A)&						    \
	self ();							    \
									    \
            inline const CAT(type,A)&					    \
	self () const;							    \
									    \
            inline CAT(type,A)&						    \
	ensureSelf (const char * = __FILE__, int = __LINE__);		    \
									    \
            inline const CAT(type,A)&					    \
	ensureSelf (const char * = __FILE__, int = __LINE__) const;	    \


#define obj_RETURNING_DEFINITIONS(type)				    	    \
	self_RETURNING_DEFINITIONS(type)				    \
									    \
            inline CAT(type,A)						    \
	makeCopy (int MakeConst = FALSE) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator + () const;						    \


	obj_RETURNING_DEFINITIONS(obj)

   };


/////////////////////////////////////////////////////////////////////////////
// class objG

CLASS_G(obj, oathCore)

	// Constructors //////////

	    inline
        objG (int IsConst = FALSE);

	    inline virtual
        ~objG ();


    public:
        // oathCore Operations //////////

	    virtual int
	isImplementedAs (void* TypeId) const
	    = 0;		// Forces it to be redefined

	    virtual void
	export (exportP&) const
	    = 0;	     	// Forces it to be redefined
	    
	    static objA
	import (importP& M);   	// Should be redefined with export

	//clearReferences;	// May be overridden to allow collection
	//setReferences;	// of circularly-referenced garbage caused
				// by objects of the given type.

        // obj Operations //////////

	    virtual int
	isEqual (const objG* O) const
	    = 0;

	    virtual objA
	makeCopy (int MakeConst) const
	    = 0;


   }; //objG


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// objG Inlines

// Constructors //////////

    inline objG::
objG (int IsConst) // IsConst = 0
   :oathCoreG(IsConst)
   {}

    inline objG::
~objG ()
   {}


/////////////////////////////////////////////////////////////////////////////
// objA Inlines

INLINES(obj, oathCore)

    inline int objA::
isConst () const
   {return guts()->isConst();}

    inline int objA::
isNil () const
   {return guts()->isNil();}

    inline int objA::
operator ! () const
   {return isNil();}

    inline int objA::
is (const objA& O) const
   {return guts() == O.guts();}

    inline int objA::
isEqual (const objA& O) const
   {return is(O) || (!isNil() && guts()->isEqual(O.guts()));}

    inline int objA::
operator == (const objA& O) const
   {return isEqual(O);}

    inline int objA::
operator != (const objA& O) const
   {return !isEqual(O);}


#define self_RETURNING_INLINES(type)					    \
									    \
    inline CAT(type,A)& CAT(type,A)::					    \
self ()									    \
   {return *this;}							    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
self () const								    \
   {return *this;}							    \
									    \
    inline CAT(type,A)& CAT(type,A)::					    \
ensureSelf (const char * File, int Line)				    \
   {if(isNil())								    \
	printAndAbort("Object is erroneously Nil!", File, Line);	    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
ensureSelf (const char * File, int Line) const				    \
   {if(isNil())								    \
	printAndAbort("Object is erroneously Nil!", File, Line);	    \
    return *this;							    \
   }									    \


#define obj_RETURNING_INLINES(type)					    \
self_RETURNING_INLINES(type)						    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
makeCopy (int MakeConst) const						    \
   {if(isNil())								    \
        return *this;							    \
    else								    \
        return (const CAT(type,A)&)(guts()->makeCopy(MakeConst));	    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator + () const							    \
   {return makeCopy();}							    \


obj_RETURNING_INLINES(obj)


/////////////////////////////////////////////////////////////////////////////
// Utility functions

    objA
callSelf (objA O);

    objA
callMakeCopy (objA O);


//***************************************************************************

#endif /* OATH_OBJ_H */
