#ifndef OATH_QUEUE_H

#define OATH_QUEUE_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  queue (queueA, queueG)
//
//  History:
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "bag.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class queueA

CLASS_A(queue, bag)

            inline objA
	front () const;

            inline objA
	operator * () const;

            inline objA
	remove () const;


#define queue_RETURNING_DEFINITIONS(type)				    \
	bag_RETURNING_DEFINITIONS(type)					    \
									    \
            inline const CAT(type,A)&					    \
	extract () const;						    \
									    \
            inline const CAT(type,A)&					    \
	extractX (objA& O) const;					    \
									    \
            inline const CAT(type,A)&					    \
	operator >> (objA& O) const;					    \


	queue_RETURNING_DEFINITIONS(queue)

   };


/////////////////////////////////////////////////////////////////////////////
// class queueG

CLASS_G(queue, bag)

	// Constructors //////////

	    inline
        queueG (int IsConst = FALSE);

    public:
        // Queue Operations //////////

	    virtual const objG*
	front () const 
	    = 0;

	    virtual const objG*
	remove ()
	    = 0;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// queueG Inlines

// Constructors //////////

    inline queueG::
queueG (int IsConst) // IsConst = FALSE)
   :bagG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// queueA Inlines

INLINES(queue, bag)

    inline objA queueA::
front () const
   {return am().guts()->front();}

    inline objA queueA::
operator * () const
   {return am().guts()->front();}

    inline objA queueA::
remove () const	
   {return am().guts()->remove();}


#define queue_RETURNING_INLINES(type)					    \
bag_RETURNING_INLINES(type)						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
extract () const							    \
   {am().guts()->remove();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
extractX (objA& O) const						    \
   {O = am().guts()->remove();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator >> (objA& O) const						    \
   {return extractX(O);}						    \


queue_RETURNING_INLINES(queue)


//***************************************************************************

#endif /* OATH_QUEUE_H */
