#ifndef OATH_SEQ_H

#define OATH_SEQ_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  seq (seqA, seqG)
//  pos (posA, posG)
//
//  History:
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "queue.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

class posA;
class posG;

/////////////////////////////////////////////////////////////////////////////
// class seqA

CLASS_A(seq, queue)

	    inline objA
	operator [] (int I) const;

	    inline objA
	operator [] (const posA& P) const;

	    inline posA
	makePos (int I = 0, int MakeConst = FALSE) const;


#define seq_RETURNING_DEFINITIONS(type)				    	    \
	queue_RETURNING_DEFINITIONS(type)				    \
									    \
	    inline CAT(type,A)						    \
        makeCopy (const posA& Start, const posA& Beyond,		    \
		  int MakeConst = FALSE) const;				    \
									    \
	    inline CAT(type,A)						    \
        makeCopy (int Start, int Beyond, int MakeConst = FALSE) const;	    \


seq_RETURNING_DEFINITIONS(seq)

   };


/////////////////////////////////////////////////////////////////////////////
// class seqG

CLASS_G(seq, queue)

	// Constructors //////////

	    inline
        seqG (int IsConst = FALSE);

    public:
        // Seq Operations //////////

	    virtual const objG*
	subscript (int I) const
	    = 0;

	    virtual const objG*
	subscript (const posG*) const
	    = 0;

	    virtual posA
	makePos (int I, int MakeConst) const
	    = 0;

	    virtual seqA
        makeSeq (const posG* Start, const posG* Beyond, int MakeConst) const
	    = 0;

	    virtual seqA
        makeSeq (int Start, int Beyond, int MakeConst) const
	    = 0;

   };


/////////////////////////////////////////////////////////////////////////////
// class posA

CLASS_A(pos, obj)

	    inline int
	isPastEnd () const;

	    inline int
	operator () () const;

	    inline seqA
	parent () const;

	    inline objA
	operator * () const;

	    inline const objA&
	find (const objA& O) const;

	    inline const objA&
	findEqual (const objA& O) const;


#define pos_RETURNING_DEFINITIONS(type)				    	    \
	obj_RETURNING_DEFINITIONS(type)					    \
									    \
            inline const CAT(type,A)&					    \
	operator ++ () const;						    \
									    \
	    inline const CAT(type,A)&					    \
	operator += (int I) const;					    \
									    \
	    inline const CAT(type,A)&					    \
	reset (const posA& P) const;					    \
									    \
	    inline const CAT(type,A)&					    \
	reset (int I = 0) const;					    \


	pos_RETURNING_DEFINITIONS(pos)

   };


/////////////////////////////////////////////////////////////////////////////
// class posG

CLASS_G(pos, obj)

	// Constructors //////////

	    inline
        posG (int IsConst = FALSE);

    public:
        // Pos Operations //////////

	    virtual int
	isPastEnd () const
	    = 0;

	    virtual const objG*
	indirection () const
	    = 0;

	    virtual void
	increment ()
	    = 0;

	    virtual void
	increment (int I)
	    = 0;

	    virtual const objG*
	find (const objG* O)
	    = 0;

	    virtual const objG*
	findEqual (const objG* O)
	    = 0;

	    virtual void
	reset (const posG* P)
	    = 0;

	    virtual void
	reset (int I)
	    = 0;

	    virtual const seqG*
	parent () const
	    = 0;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// seqG Inlines

// Constructors //////////

    inline seqG::
seqG (int IsConst) // IsConst = FALSE)
   :queueG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// seqA Inlines

INLINES(seq, queue)

    inline objA seqA::
operator [] (int I) const
   {return am().guts()->subscript(I);}

    inline objA seqA::
operator [] (const posA& P) const
   {return am().guts()->subscript(P.guts());}

    inline posA seqA::
makePos (int I, int MakeConst) const
   {return am().guts()->makePos(I, MakeConst);}


#define seq_RETURNING_INLINES(type)					    \
queue_RETURNING_INLINES(type)						    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
makeCopy (const posA& Start, const posA& Beyond = posA::Nil,		    \
			     int MakeConst) const			    \
   {return (CAT(type,A)&)						    \
	   am().guts()->makeSeq(Start.guts(), Beyond.guts(),MakeConst);}    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
makeCopy (int Start, int Beyond, int MakeConst) const			    \
   {return (CAT(type,A)&)am().guts()->makeSeq(Start, Beyond, MakeConst);}   \


seq_RETURNING_INLINES(seq)


/////////////////////////////////////////////////////////////////////////////
// posG Inlines

// Constructors //////////

    inline posG::
posG (int IsConst) // IsConst = FALSE)
   :objG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// posA Inlines

INLINES(pos, obj)

    inline int posA::
isPastEnd () const
   {return am().guts()->isPastEnd();}

    inline int posA::
operator () () const
   {return !isPastEnd();}

    inline seqA posA::
parent () const
   {return am().guts()->parent();}

    inline objA posA::
operator * () const
   {return am().guts()->indirection();}

    inline const objA& posA::
find (const objA& O) const
   {if(am().guts()->find(O.guts()))
        return O;
    else
        return objA::Nil;
   }

    inline const objA& posA::
findEqual (const objA& O) const
   {if(am().guts()->findEqual(O.guts()))
        return O;
    else
        return objA::Nil;
   }


#define pos_RETURNING_INLINES(type)					    \
obj_RETURNING_INLINES(type)						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator ++ () const							    \
   {am().guts()->increment();						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator += (int I) const						    \
   {am().guts()->increment(I);						    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
reset (const posA& P) const						    \
   {am().guts()->reset(P.guts());					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
reset (int I) const							    \
   {am().guts()->reset(I);						    \
    return *this;							    \
   }									    \


pos_RETURNING_INLINES(pos)


//***************************************************************************

#endif /* OATH_SEQ_H */
