#ifndef OATH_SET_H

#define OATH_SET_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  set (setA, setG)
//
//  History:
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    12/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "bag.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class setA

CLASS_A(set, bag)


#define set_RETURNING_DEFINITIONS(type)	    		    	    	    \
	bag_RETURNING_DEFINITIONS(type)					    \
									    \
            inline const CAT(type,A)&					    \
        remove (const objA& O) const;					    \
									    \
	    inline const CAT(type,A)&					    \
	operator -= (const objA& O) const;				    \
									    \
            inline const CAT(type,A)&					    \
        subtract (const setA& S) const;					    \
									    \
	    inline const CAT(type,A)&					    \
	operator -= (const setA& S) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator - (const setA& S) const;				    \
									    \
            inline const CAT(type,A)&					    \
        intersect (const setA& S) const;				    \
									    \
	    inline const CAT(type,A)&					    \
	operator &= (const setA& S) const;				    \
									    \
	    inline CAT(type,A)						    \
	operator & (const setA& S) const;				    \


	set_RETURNING_DEFINITIONS(set)

   };


/////////////////////////////////////////////////////////////////////////////
// class setG

CLASS_G(set, bag)

	// Constructors //////////

	    inline
        setG (int IsConst = FALSE);

    public:
        // set Operations //////////

	    virtual void
	remove (const objG* O)
	    = 0;

	    virtual void
	subtract (const setG* S)
	    = 0;

	    virtual void
	intersect (const setG* S)
	    = 0;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// setG Inlines

// Constructors //////////

    inline setG::
setG (int IsConst) // IsConst = FALSE
   :bagG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// setA Inlines

INLINES(set, bag)

#define set_RETURNING_INLINES(type)		    		    	    \
bag_RETURNING_INLINES(type)						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
remove (const objA& O) const						    \
   {am().guts()->remove(O.guts());					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator -= (const objA& O) const					    \
   {return remove(O);}							    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
subtract (const setA& S) const						    \
   {am().guts()->subtract(S.guts());					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator -= (const setA& S) const					    \
   {return subtract(S);}						    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator - (const setA& S) const					    \
   {return makeCopy().subtract(S);}					    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
intersect (const setA& S) const						    \
   {am().guts()->intersect(S.guts());					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator &= (const setA& S) const					    \
   {return intersect(S);}						    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator & (const setA& S) const					    \
   {return makeCopy().intersect(S);}					    \


set_RETURNING_INLINES(set)


//***************************************************************************

#endif /* OATH_SET_H */
