#ifndef OATH_SLNODEP_H

#define OATH_SLNODEP_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  slNodeP
//
//  History:
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************
//  slNodeP (Singly-Linked Node Auxiliary Type)
//
//  slNodeP is an auxiliary structure used to implement the pdlQueue type.
//  slNodeP is NOT an OATH type!  slNodeP is a class containing two data
//  items:  an objA and a pointer to the next slNodeP.
//
//  The following operations are defined on slNodeP's:
//    thisObj()         --- return a pointer to the objG in this slNodeP
//    next()		--- return a pointer to the next slNodeP
//    nextObj()         --- return a pointer to the objG in the next slNodeP
//    insertAfter(Obj)  --- construct and insert a slNodeP that points to Obj
//			    as the next node
//    removeNext()	--- remove and return the next slNodeP
//    deleteNext()	--- remove and delete the next slNodeP
//
//***************************************************************************

#include "obj.h"

class slNodeP;

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// class slNodeP

class slNodeP

   {public:

   	slNodeP* Next;
	objA     Obj;

   	slNodeP ()
	   :Next(0), Obj()
	   {}

	slNodeP (slNodeP* iNext, const objG* iObj)
	   :Next(iNext), Obj(iObj)
	   {}

	    objA&
	thisObj ()
	   {return Obj;}

	    slNodeP*
	next ()
	   {return Next;}

	    objA&
	nextObj ()
	   {assumed(Next, "Attempted slNodeP::nextObj() with no Next"); 
	    return Next->Obj;
	   }

	    slNodeP*
	insertAfter (const objG* NewObj)
	   {return Next = new slNodeP (Next, NewObj);}

	    slNodeP*
	removeNext ()
	   {assumed(Next, "Attempted slNodeP::removeNext() with no Next");
	    slNodeP *DLN = Next;
	    Next = Next->Next;
	    return DLN;
	   }

	    void
	deleteNext ()
	   {assumed(Next, "Attempted slNodeP::deleteNext() with no Next");
	    slNodeP *DLN = Next;
	    Next = Next->Next;
	    delete DLN;
	   }

    // Dynamic Allocation
    //   for use on systems with expensive malloc

#ifdef OATH_HANDLE_NEW

	static int      AllocateCount;
	static slNodeP *FreeList;

	    static void
	allocate ();

	    static void*
        operator new (size_t)
           {if(!FreeList)
	       allocate(); 
	    slNodeP *DLN = FreeList;
	    FreeList = FreeList->Next;
            return DLN;
           }

            static void
        operator delete (void *DLN, size_t)
           {((slNodeP*)DLN)->Next = FreeList;
            FreeList = (slNodeP*)DLN;
           }

#endif

   };

/***************************************************************************/

#endif  /* OATH_SLNODEP_H */
