#ifndef STNODEP_H

#define STNODEP_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  stNodeP
//
//  History:
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************
//  stNodeP (String Table Node)
//
//  stNodeP is an auxiliary structure used to implement the stringGable
//  type.  stNodeP is NOT an obj!  stNodeP is a class containing
//  three data items:  a const_minStringA, a const_objA, and a pointer to
//  the next stNodeP.
//
//  The following operations are defined on stNodeP's:
//    contains(S)	--- Return TRUE if this const_stringTokenA is equal to
//			    the stringToken S or if Next->contains(S).
//    assoc(S)		--- Return this const_objA if this const_stringTokenA is
//			    equal to the stringToken S or return Next->assoc(S).
//    reassoc(S, O)	--- If this const_stringTokenA is equal to S, then 
//			    change this const_objA to O and return TRUE.
//			    Otherwise, if Next, return Next->reassoc(S, O).
//    lookup(Str)	--- If the string Str is equal to this stringToken's
//			    string, then return this stringToken.  Otherwise
//			    return Next->lookup(Str).
//    lookup(Chars)     --- Same as above for char* Chars.
//    lookup(Start, Bey)--- Same as above for stringPos's Start and Bey.
//
//
//***************************************************************************

// stringTokenG and stringTokenA MUST be defined prior to here

/////////////////////////////////////////////////////////////////////////////
// stNodeP Class Definition

class stNodeP
   {friend class stringTableG;
    friend class stringTableA;

        // Members //////////

   	stNodeP *     Next;
	stringTokenA  Token;
	objA          Obj;

        // Constructors //////////

   	stNodeP ()
	   :Next(0), Token(), Obj()
	   {ensure(FALSE, "This shouldn't be called!");}

	stNodeP (const stNodeP&)
	   :Next(0), Token(), Obj()
	   {ensure(FALSE, "This shouldn't be called!");}

	stNodeP (const stringTokenG* S, const objG* O, stNodeP* N)
	   :Next(N), Token(S), Obj(O)
	   {assumed(!!Token, "Nil Token inserted in Table!!");}

	// Operations //////////

   	    void
	printDiagnostic(ostream&, int)
	   {}

	    int
        contains (const stringTokenG* S) const
	   {return S->is(Token.guts()) 
		   ? TRUE
		   : (Next ? Next->contains(S) : FALSE);
	   }

	    const objG*
	remove (const stringTokenG* S, stNodeP*& P)
	   {const objG* R;
	    if(S->is(Token.guts()))
	       {P = Next;
	        R = Obj.guts();
		delete this;
	       }
	    else if(Next)
	        R = Next->remove(S, Next);
	    else
	        R = objG::Nil;
	    return R;
	   }

	    const objG*
        assoc(const stringTokenG* S) const
	   {return S->is(Token.guts())
		   ? Obj.guts()
		   : (Next ? Next->assoc(S) : objG::Nil);
	   }

	    int
        reassoc(const stringTokenG* S, const objG* O)
	   {return S->is(Token.guts()) 
		   ? (Obj = O, TRUE) 
		   : (Next ? Next->reassoc(S, O) : FALSE);
	   }

	    const stringTokenG*
        lookup (const stringG* S) const
	   {return Token.string().guts()->isEqual(S) 
		   ? Token.guts()
		   : (Next ? Next->lookup(S) : stringTokenG::Nil);
	   }

	    const stringTokenG*
        lookup (const char * C) const
	   {return Token.string().guts()->isEqual(C)
		   ? Token.guts()
		   : (Next ? Next->lookup(C) : stringTokenG::Nil);
	   }

	    const stringTokenG*
        lookup (const stringPosG* Start, const stringPosG* Beyond) const
	   {return Token.string().guts()->isEqual(Start, Beyond) 
		   ? Token.guts()
		   : (Next ? Next->lookup(Start, Beyond) : stringTokenG::Nil);
	   }

	    const stringTokenG*
        lookupAnyCase (const stringG* S) const
	   {return Token.string().guts()->isEqualAnyCase(S) 
		   ? Token.guts()
		   : (Next ? Next->lookupAnyCase(S) : stringTokenG::Nil);
	   }

	    const stringTokenG*
        lookupAnyCase (const char * C) const
	   {return Token.string().guts()->isEqualAnyCase(C) 
		   ? Token.guts()
		   : (Next ? Next->lookupAnyCase(C) : stringTokenG::Nil);
	   }

	    const stringTokenG*
        lookupAnyCase (const stringPosG* Start, const stringPosG* Beyond) const
	   {return Token.string().guts()->isEqualAnyCase(Start, Beyond) 
		   ? Token.guts()
		   : (Next ? Next->lookupAnyCase(Start, Beyond)
			   : stringTokenG::Nil);
	   }

   };

/***************************************************************************/

#endif  /* STNODEP_H */
