#ifndef OATH_STRING_H

#define OATH_STRING_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  string (stringA, stringG)
//  stringPos (stringPosA, stringPosG)
//
//  History:
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "list.h"

#include "character.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

class stringPosA;
class stringPosG;

/////////////////////////////////////////////////////////////////////////////
// class stringA

CLASS_A(string, list)

	    inline stringPosA
	makePos (int I = 0, int MakeConst = FALSE) const;

	    inline int
	isEqual (const char *) const;

	    inline int
	isEqual (const stringPosA&, const stringPosA&) const;

	    inline int
	isEqualAnyCase (const stringA&) const;

	    inline int
	isEqualAnyCase (const char *) const;

	    inline int
	isEqualAnyCase (const stringPosA&, const stringPosA&) const;

	    inline int
	operator <= (const stringA&) const;

	    inline int
	operator <  (const stringA&) const;

	    inline int
	operator >  (const stringA&) const;

	    inline int
	operator >= (const stringA&) const;

            inline const char *
	charStar () const;

            inline void
	charStarX (char * C, int N) const;

	    inline int
	hash () const; // uses K = 4

	    inline int
	hash (int K) const;

	    inline int
	hashLower () const; // uses K = 4

	    inline int
	hashLower (int K) const;

	    inline int
	hashUpper () const; // uses K = 4

	    inline int
	hashUpper (int K) const;

	    inline const bagA&
	applyX (objA (*F)(objA), const bagA& B) const;

	    inline const bagA&
	applyX (bagA (*F)(objA), const bagA& B) const;

            inline const stringA&
        applyX (characterA (*F)(characterA), const stringA& S) const;

            inline const stringA&
        applyX (stringA (*F)(characterA), const stringA& S) const;


#define string_RETURNING_DEFINITIONS(type)		    	    	    \
	list_RETURNING_DEFINITIONS(type)				    \
									    \
            inline CAT(type,A)						    \
        make (characterA (*F)(characterA)) const;			    \
									    \
            inline CAT(type,A)						    \
        make (stringA (*F)(characterA)) const;				    \
									    \
            inline CAT(type,A)						    \
        lowercase () const;						    \
									    \
            inline CAT(type,A)						    \
        uppercase () const;						    \
									    \
            inline CAT(type,A)						    \
        makeLower (int MakeConst = FALSE) const;			    \
									    \
            inline CAT(type,A)						    \
        makeUpper (int MakeConst = FALSE) const;			    \
									    \
	    inline const CAT(type,A)&					    \
	operator << (const characterA& C) const;			    \
									    \
	    inline const CAT(type,A)&					    \
	operator << (char C) const;					    \
									    \
	    inline const CAT(type,A)&					    \
	operator << (const stringA& S) const;				    \
									    \
	    inline const CAT(type,A)&					    \
	operator << (const char * S) const;				    \


	string_RETURNING_DEFINITIONS(string)

   };


/////////////////////////////////////////////////////////////////////////////
// class stringG

CLASS_G(string, list)

    friend stringPosG;

	// Constructors //////////

	    inline
        stringG (int IsConst = FALSE);

    public:
        // obj Operations //////////

	    virtual int
	isEqual (const objG*) const
	    = 0;

        // string Operations //////////

	    virtual int
	isEqual (const char * C) const
	    = 0;

	    virtual int
	isEqual (const stringPosG*, const stringPosG*) const
	    = 0;

	    virtual int
	isEqualAnyCase (const stringG*) const
	    = 0;

	    virtual int
	isEqualAnyCase (const char * C) const
	    = 0;

	    virtual int
	isEqualAnyCase (const stringPosG*, const stringPosG*) const
	    = 0;

	    virtual int
	lessThan (const stringG* S) const
	    = 0;

	    virtual int
	moreThan (const stringG* S) const
	    = 0;

	    virtual void
	stringInsert (const characterG* C)
	    = 0;

	    virtual void
	stringInsert (char C)
	    = 0;

	    virtual void
	stringAppend (const stringG* S)
	    = 0;

	    virtual void
	stringAppend (const char * S)
	    = 0;

	    virtual const char *
	charStar () const
	    = 0;

	    virtual void
	charStarX (char * C, int N) const
	    = 0;

	    virtual int
	hash () const
	    = 0;

	    virtual int
	hash (int K) const
	    = 0;

	    virtual int
	hashLower () const
	    = 0;

	    virtual int
	hashLower (int K) const
	    = 0;

	    virtual int
	hashUpper () const
	    = 0;

	    virtual int
	hashUpper (int K) const
	    = 0;

	    virtual stringG*
	applyStringX (characterA (*)(characterA), stringG* S) const
	    = 0;

	    virtual stringG*
	applyStringX (stringA (*)(characterA), stringG* S) const
	    = 0;

	    virtual void
	lowercase ()
	    = 0;

	    virtual void
	uppercase ()
	    = 0;

	    virtual stringA
	makeLower (int MakeConst) const
	    = 0;

	    virtual stringA
	makeUpper (int MakeConst) const
	    = 0;

	    virtual ostream&
	ostreamInsertion (ostream& F) const
	    = 0;

	    virtual istream&
	istreamExtraction (istream& F)
	    = 0;


    // Static Functions

	    static int
        hash (const char * C);

	    static int
        hash (const stringPosG* Start, const stringPosG* Beyond);

	    static int
        hashLower (const char * C);

	    static int
        hashLower (const stringPosG* Start, const stringPosG* Beyond);

	    static int
        hashUpper (const char * C);

	    static int
        hashUpper (const stringPosG* Start, const stringPosG* Beyond);

   };


/////////////////////////////////////////////////////////////////////////////
// class stringPosA

CLASS_A(stringPos, listPos)

	    inline characterA
	operator * () const;

	    inline int
        skipSpaces () const;

	    inline int
        canMatch (const stringA& S) const;

	    inline int
        canMatchAnyCase (const stringA& S) const;

	    inline int
        canMatch (const char * S) const;

	    inline int
        canMatchAnyCase (const char * S) const;

	    inline const stringA&
        match (const stringA& S) const;

	    inline const stringA&
        find (const stringA& S) const;

	    inline const stringA&
        findMatch (const stringA& S) const;

	    inline const stringA&
        matchAnyCase (const stringA& S) const;

	    inline const stringA&
        findAnyCase (const stringA& S) const;

	    inline const stringA&
        findMatchAnyCase (const stringA& S) const;

	    inline const char *
        match (const char * S) const;

	    inline const char *
        find (const char * S) const;

	    inline const char *
        findMatch (const char * S) const;

	    inline const char *
        matchAnyCase (const char * S) const;

	    inline const char *
        findAnyCase (const char * S) const;

	    inline const char *
        findMatchAnyCase (const char * S) const;


#define stringPos_RETURNING_DEFINITIONS(type)		    	    	    \
	listPos_RETURNING_DEFINITIONS(type)				    \


	stringPos_RETURNING_DEFINITIONS(stringPos)

   };


/////////////////////////////////////////////////////////////////////////////
// class stringPosG

CLASS_G(stringPos, listPos)

    friend stringG;

	// Constructors //////////

	    inline
        stringPosG (int IsConst = FALSE);

    public:
        // stringPos Operations //////////

	    virtual const characterG*
	charIndirection () const
	    = 0;

	    virtual int 
	skipSpaces ()
	    = 0;

	    virtual const stringG*
	match (const stringG* S)
	    = 0;

	    virtual int
	canMatch (const stringG* S) const
	    = 0;

	    virtual const stringG*
	findString (const stringG* S)
	    = 0;

	    virtual const stringG*
	findMatch (const stringG* S)
	    = 0;

	    virtual const stringG*
	matchAnyCase (const stringG* S)
	    = 0;

	    virtual int
	canMatchAnyCase (const stringG* S) const
	    = 0;

	    virtual const stringG*
	findAnyCase (const stringG* S)
	    = 0;

	    virtual const stringG*
	findMatchAnyCase (const stringG* S)
	    = 0;

	    virtual const char *
	match (const char * S)
	    = 0;

	    virtual int
	canMatch (const char * S) const
	    = 0;

	    virtual const char *
	findString (const char * S)
	    = 0;

	    virtual const char *
	findMatch (const char * S)
	    = 0;

	    virtual const char *
	matchAnyCase (const char * S)
	    = 0;

	    virtual int
	canMatchAnyCase (const char * S) const
	    = 0;

	    virtual const char *
	findAnyCase (const char * S)
	    = 0;

	    virtual const char *
	findMatchAnyCase (const char * S)
	    = 0;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// stringG Inlines

// Constructors //////////

    inline stringG::
stringG (int IsConst) // IsConst = FALSE)
   :listG(IsConst)
   {}

/////////////////////////////////////////////////////////////////////////////
// stringA Inlines

INLINES(string, list)

    inline stringPosA stringA::
makePos (int I, int MakeConst) const
   {return (stringPosA&)(am().guts()->makePos(I, MakeConst));}

    inline int stringA::
isEqual (const char * C) const
   {return am().guts()->isEqual(C);}

    inline int stringA::
isEqual (const stringPosA& Start, const stringPosA& Beyond) const
   {return am().guts()->isEqual(Start.guts(), Beyond.guts());}

    inline int stringA::
isEqualAnyCase (const stringA& S) const
   {return am().guts()->isEqualAnyCase(S.guts());}

    inline int stringA::
isEqualAnyCase (const char * C) const
   {return am().guts()->isEqualAnyCase(C);}

    inline int stringA::
isEqualAnyCase (const stringPosA& Start, const stringPosA& Beyond) const
   {return am().guts()->isEqualAnyCase(Start.guts(), Beyond.guts());}

    inline int stringA::
operator <= (const stringA& S) const
   {return !am().guts()->moreThan(S.guts());}

    inline int stringA::
operator <  (const stringA& S) const
   {return am().guts()->lessThan(S.guts());}

    inline int stringA::
operator >  (const stringA& S) const
   {return am().guts()->moreThan(S.guts());}

    inline int stringA::
operator >= (const stringA& S) const
   {return !am().guts()->lessThan(S.guts());}

    inline const char * stringA::
charStar () const
   {return am().guts()->charStar();}

    inline void stringA::
charStarX (char * C, int N) const
   {am().guts()->charStarX(C, N);}

    inline int stringA::
hash () const // uses K = 4
   {return am().guts()->hash();}

    inline int stringA::
hash (int K) const
   {return am().guts()->hash(K);}

    inline int stringA::
hashLower () const // uses K = 4
   {return am().guts()->hashLower();}

    inline int stringA::
hashLower (int K) const
   {return am().guts()->hashLower(K);}

    inline int stringA::
hashUpper () const // uses K = 4
   {return am().guts()->hashUpper();}

    inline int stringA::
hashUpper (int K) const
   {return am().guts()->hashUpper(K);}

    inline const bagA& stringA::
applyX (objA (*F)(objA), const bagA& B) const
   {return bagA::applyX(F, B);}

    inline const bagA& stringA::
applyX (bagA (*F)(objA), const bagA& B) const
   {return bagA::applyX(F, B);}

    inline const stringA& stringA::
applyX (characterA (*F)(characterA), const stringA& S) const
   {am().guts()->applyStringX(F, S.guts());
    return S;
   }

    inline const stringA& stringA::
applyX (stringA (*F)(characterA), const stringA& S) const
   {am().guts()->applyStringX(F, S.guts());
    return S;
   }


#define string_RETURNING_INLINES(type)		    	    	    	    \
list_RETURNING_INLINES(type)						    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
make (characterA (*F)(characterA)) const				    \
   {CAT(type,A) Temp = makeEmpty();					    \
    applyX(F, Temp);							    \
    return Temp;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
make (stringA (*F)(characterA)) const					    \
   {CAT(type,A) Temp = makeEmpty();					    \
    applyX(F, Temp);							    \
    return Temp;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
lowercase () const							    \
   {am().guts()->lowercase();						    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
uppercase () const							    \
   {am().guts()->uppercase();						    \
    return *this;							    \
   }									    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
makeLower (int MakeConst) const						    \
   {return (CAT(type,A)&)(am().guts()->makeLower(MakeConst));}		    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
makeUpper (int MakeConst) const						    \
   {return (CAT(type,A)&)(am().guts()->makeUpper(MakeConst));}		    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator << (const characterA& C) const					    \
   {am().guts()->stringInsert(C.guts());				    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator << (char C) const						    \
   {am().guts()->stringInsert(C);					    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator << (const stringA& S) const					    \
   {am().guts()->stringAppend(S.guts());				    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
operator << (const char * S) const					    \
   {am().guts()->stringAppend(S);					    \
    return *this;							    \
   }									    \


string_RETURNING_INLINES(string)


/////////////////////////////////////////////////////////////////////////////
// stringPosG Inlines

// Constructors //////////

    inline stringPosG::
stringPosG (int IsConst) // IsConst = FALSE)
   :listPosG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// stringPosA Inlines

INLINES(stringPos, listPos)

    inline characterA stringPosA::
operator * () const
   {return am().guts()->charIndirection();}

    inline int stringPosA::
skipSpaces () const
   {return am().guts()->skipSpaces();}

    inline int stringPosA::
canMatch (const stringA& S) const
   {return am().guts()->canMatch(S.guts());}

    inline int stringPosA::
canMatchAnyCase (const stringA& S) const
   {return am().guts()->canMatchAnyCase(S.guts());}

    inline int stringPosA::
canMatch (const char * S) const
   {return am().guts()->canMatch(S);}

    inline int stringPosA::
canMatchAnyCase (const char * S) const
   {return am().guts()->canMatchAnyCase(S);}

    inline const stringA& stringPosA::
match (const stringA& S) const
   {return am().guts()->match(S.guts()) ? S : stringA::Nil;}

    inline const stringA& stringPosA::
find (const stringA& S) const
   {return am().guts()->findString(S.guts()) ? S : stringA::Nil;}

    inline const stringA& stringPosA::
findMatch (const stringA& S) const
   {return am().guts()->findMatch(S.guts()) ? S : stringA::Nil;}

    inline const stringA& stringPosA::
matchAnyCase (const stringA& S) const
   {return am().guts()->matchAnyCase(S.guts()) ? S : stringA::Nil;}

    inline const stringA& stringPosA::
findAnyCase (const stringA& S) const
   {return am().guts()->findAnyCase(S.guts()) ? S : stringA::Nil;}

    inline const stringA& stringPosA::
findMatchAnyCase (const stringA& S) const
   {return am().guts()->findMatchAnyCase(S.guts()) ? S : stringA::Nil;}

    inline const char * stringPosA::
match (const char * S) const
   {return am().guts()->match(S);}

    inline const char * stringPosA::
find (const char * S) const
   {return am().guts()->findString(S);}

    inline const char * stringPosA::
findMatch (const char * S) const
   {return am().guts()->findMatch(S);}

    inline const char * stringPosA::
matchAnyCase (const char * S) const
   {return am().guts()->matchAnyCase(S);}

    inline const char * stringPosA::
findAnyCase (const char * S) const
   {return am().guts()->findAnyCase(S);}

    inline const char * stringPosA::
findMatchAnyCase (const char * S) const
   {return am().guts()->findMatchAnyCase(S);}


#define stringPos_RETURNING_INLINES(type)		    	    	    \
listPos_RETURNING_INLINES(type)						    \


stringPos_RETURNING_INLINES(stringPos)


/////////////////////////////////////////////////////////////////////////////
// Other Operations

    inline ostream&       
operator << (ostream& F, const stringA& S)
   {return S.guts()->ostreamInsertion(F);}

    inline istream&
operator >> (istream& F, const stringA& S)
   {return S.guts()->istreamExtraction(F);}


//***************************************************************************

#endif /* OATH_STRING_H */
