#ifndef OATH_TABLE_H

#define OATH_TABLE_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  table (tableA, tableG)
//
//  History:
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "bag.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class tableA

CLASS_A(table, bag)

            inline objA
        remove (const objA& O) const;

            inline objA
        assoc (const objA& O) const;


#define table_RETURNING_DEFINITIONS(type)	    		    	    \
	bag_RETURNING_DEFINITIONS(type)					    \
									    \
            inline const CAT(type,A)&					    \
        insert (const objA& O, const objA& A) const;			    \
									    \
            inline const CAT(type,A)&					    \
        reassoc (const objA& O, const objA& A) const;			    \
									    \
            inline const CAT(type,A)&					    \
        extract (const objA& O) const;					    \


	table_RETURNING_DEFINITIONS(table)

   };


/////////////////////////////////////////////////////////////////////////////
// class tableG

CLASS_G(table, bag)

	// Constructors //////////

	    inline
        tableG (int IsConst = FALSE);

    public:
        // table Operations //////////

	    virtual void
	insertAssoc (const objG* O, const objG* A)
	    = 0;

	    virtual void
	reassoc (const objG* O, const objG* A)
	    = 0;

	    virtual const objG*
	remove (const objG* O)
	    = 0;

	    virtual const objG*
	assoc (const objG* O) const
	    = 0;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// tableG Inlines

// Constructors //////////

    inline tableG::
tableG (int IsConst) // IsConst = FALSE
   :bagG(IsConst)
   {}


/////////////////////////////////////////////////////////////////////////////
// tableA Inlines

INLINES(table, bag)

    inline objA tableA::
remove (const objA& O) const
   {return am().guts()->remove(O.guts());}

    inline objA tableA::
assoc (const objA& O) const
   {return am().guts()->assoc(O.guts());}


#define table_RETURNING_INLINES(type)		    		    	    \
bag_RETURNING_INLINES(type)						    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
insert (const objA& O, const objA& A) const				    \
   {am().guts()->insertAssoc(O.guts(), A.guts());			    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
reassoc (const objA& O, const objA& A) const				    \
   {am().guts()->reassoc(O.guts(), A.guts());				    \
    return *this;							    \
   }									    \
									    \
    inline const CAT(type,A)& CAT(type,A)::				    \
extract (const objA& O) const						    \
   {am().guts()->remove(O.guts());					    \
    return *this;							    \
   }									    \


table_RETURNING_INLINES(table)


//***************************************************************************

#endif /* OATH_TABLE_H */
