#ifndef OATH_TOKEN_H

#define OATH_TOKEN_H

//***************************************************************************
//             OATH :: Object-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  token (tokenA, tokenG)
//
//  History:
//    07/91  Brian M Kennedy  Separate out localToken
//    07/91  Brian M Kennedy  import, export, typeRegister
//    06/91  Brian M Kennedy  New macros & format; remove printDiagnostic
//    10/90  Brian M Kennedy  Major Rewrite
//    02/90  Brian M Kennedy  Original
//
//***************************************************************************

#include "obj.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Class Definitions

/////////////////////////////////////////////////////////////////////////////
// class tokenA

CLASS_A(token, obj)

	// Operations //////////

	    inline int
	isEqual (const objA& O) const;

	    inline int
	operator == (const objA& O) const;

	    inline int
	operator != (const objA& O) const;


#define token_RETURNING_DEFINITIONS(type)				    \
	obj_RETURNING_DEFINITIONS(type)					    \


	token_RETURNING_DEFINITIONS(token)

   };


/////////////////////////////////////////////////////////////////////////////
// class tokenG

CLASS_G(token, obj)

	// Constructors //////////

	    inline
        tokenG ();

    public:
        // obj Operations //////////

            inline virtual int
        isEqual (const objG* O) const;

            inline virtual objA
        makeCopy (int) const;

   };


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Inline Definitions

/////////////////////////////////////////////////////////////////////////////
// tokenG Inlines

// Constructors //////////

    inline tokenG::
tokenG ()
   :objG(TRUE)
   {}

// obj Operations //////////

    inline int tokenG::
isEqual (const objG* O) const
   {return is(O);}

    inline objA tokenG:: 
makeCopy (int) const
   {return this;}

/////////////////////////////////////////////////////////////////////////////
// tokenA Inlines

INLINES(token, obj)

// Operations //////////

    inline int tokenA::
isEqual (const objA& O) const
   {return is(O);}

    inline int tokenA::
operator == (const objA& O) const
   {return is(O);}

    inline int tokenA::
operator != (const objA& O) const
   {return !is(O);}


#define token_RETURNING_INLINES(type)					    \
self_RETURNING_INLINES(type)						    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
makeCopy (int) const							    \
   {return *this;}							    \
									    \
    inline CAT(type,A) CAT(type,A)::					    \
operator + () const							    \
   {return *this;}							    \


token_RETURNING_INLINES(token)


//***************************************************************************

#endif /* OATH_TOKEN_H */
