#ifndef OATH_TYPEREGISTERP_H

#define OATH_TYPEREGISTERP_H

//***************************************************************************
//             OATH :: TypeRegisterPect-oriented Abstract Type Hierarchy
//***************************************************************************
//
//  Copyright (C) 1991, 1990  Texas Instruments Incorporated
//  Permission is granted to any individual or institution
//  to use, copy, modify, and distribute this software,
//  provided that this complete copyright and permission notice
//  is maintained, intact, in all copies and supporting documentation.
//
//  Texas Instruments Incorporated provides this software "as is"
//  without express or implied warranty.
//
//***************************************************************************
//  typeRegisterP (private class for registering types)
//
//  History:
//    07/91  Brian M Kennedy  Original
//
//***************************************************************************

#include "ensure.h"

class ostream;
class istream;

class exportP;
class importP;

class oathCoreG;
class oathCoreA;

class objA;

typedef objA (*importF)(importP&);


/////////////////////////////////////////////////////////////////////////////

#define OATH_TYPE_REGISTRY_SIZE 297 // This is based on registerType function
				    // which adds the first four characters
				    // minus 48 (the ASCII value of '0').
				    // Given 'z' is the largest type name
				    // character, (122-48)*4+1 = 297.

/////////////////////////////////////////////////////////////////////////////
// class typeRegisterP

class typeRegisterP
   {friend class exportP;
    friend class importP;
    friend class oathCoreA;
    friend class oathCoreG;

    private:
        // Members //////////
        const char *    TypeName;
        importF         Import;
        typeRegisterP * Next;

	// Constructors //////////
	typeRegisterP ()
	   :TypeName(0), Import(0), Next(0)
	   {ensure(0, "Default constructor for typeRegisterP called!");}

        typeRegisterP (const typeRegisterP&)
	   :TypeName(0), Import(0), Next(0)
	   {ensure(0, "Default constructor for typeRegisterP called!");}

        typeRegisterP (const char* ITypeName, importF IImport,
		       typeRegisterP* INext = 0)
           :TypeName(ITypeName), Import(IImport), Next(INext)
           {ensure(TypeName && Import, "Bogus typeRegisterP created!");}


	// Static Operations //////////

	    static const char*
        registerType (const char* Name, importF Import);

	    static const char*
        readType (istream& S);

	    static ostream&
        writeType (ostream& S, const char* TypeName);

	    static typeRegisterP*
        findRegister (const char* TypeName);

	    static int
        hashTypeName (const char* TypeName);


	// Static Members //////////

        static typeRegisterP*  TypeRegistry [OATH_TYPE_REGISTRY_SIZE];

   };

//***************************************************************************

#endif /* OATH_TYPEREGISTERP_H */
